# -*- coding: utf8 -*-
'''
==============
ParameterVault
==============

A plugin that allows the user to quickly store parameter grid values and the corresponding
simulation to compare different fits agains each other. 

Written by Artur Glavic
Last Changes 04/22/15
'''

import os, sys
from copy import deepcopy
import wx
from .. import add_on_framework as framework
from genx.parameters import Parameters
import genx.images as img
from wx.lib.mixins.listctrl import ListCtrlAutoWidthMixin

class Plugin(framework.Template):
    _refplugin=None

    @property
    def refplugin(self):
        # check if reflectivity plugin is None or destoryed, try to connect
        if not self._refplugin:
            self._init_refplugin()
        return self._refplugin

    def __init__(self, parent):
        framework.Template.__init__(self, parent)
        self.parent=parent

        parameter_panel=self.NewDataFolder('Vault')
        sizer=wx.BoxSizer(wx.HORIZONTAL)
        self.parameter_panel=wx.Panel(parameter_panel)
        self.create_parameter_list()
        sizer.Add(self.parameter_panel, 1, wx.EXPAND | wx.GROW | wx.ALL)
        parameter_panel.SetSizer(sizer)
        parameter_panel.Layout()

        menu=self.NewMenu('P. Vault')
        self.mb_recalc_all=wx.MenuItem(menu, wx.NewId(),
                                       "Recalculate items",
                                       "Run simulations for each parameter set again and calculate FOM",
                                       wx.ITEM_NORMAL)
        menu.Append(self.mb_recalc_all)
        self.mb_auto_store=wx.MenuItem(menu, wx.NewId(),
                                       "Save tables",
                                       "Save a parameters table file each time you add something to the vault",
                                       wx.ITEM_CHECK)
        menu.Append(self.mb_auto_store)
        self.mb_load_all=wx.MenuItem(menu, wx.NewId(),
                                     "Recall saved tables",
                                     "Load all table parameters previously saved",
                                     wx.ITEM_NORMAL)
        menu.Append(self.mb_load_all)
        self.mb_auto_store.Check(False)

        self.parent.Bind(wx.EVT_MENU, self.OnRecalcAll, self.mb_recalc_all)
        self.parent.Bind(wx.EVT_MENU, self.OnAutoStore, self.mb_auto_store)
        self.parent.Bind(wx.EVT_MENU, self.OnLoadAll, self.mb_load_all)

    def _init_refplugin(self):
        ph=self.parent.plugin_control.plugin_handler
        if 'Reflectivity' in ph.loaded_plugins:
            # connect to the reflectivity plugin for layer creation
            self._refplugin=ph.loaded_plugins['Reflectivity']
        else:
            dlg=wx.MessageDialog(self.materials_panel, 'Reflectivity plugin must be loaded',
                                 caption='Information',
                                 style=wx.OK | wx.ICON_WARNING)
            dlg.ShowModal()
            dlg.Destroy()
            self._refplugin=None

    def create_toolbar(self):
        dpi_scale_factor=wx.GetApp().dpi_scale_factor
        tb_bmp_size=int(dpi_scale_factor*20)
        size=wx.Size(tb_bmp_size, tb_bmp_size)

        self.bitmap_button_add=wx.BitmapButton(self.tool_panel, -1
                                               , wx.Bitmap(img.getaddImage().Scale(tb_bmp_size, tb_bmp_size)),
                                               size=size, style=wx.NO_BORDER)
        self.bitmap_button_add.SetToolTip('Add current parameters to Vault')
        self.bitmap_button_delete=wx.BitmapButton(self.tool_panel, -1
                                                  , wx.Bitmap(img.getdeleteImage().Scale(tb_bmp_size, tb_bmp_size)),
                                                  size=size, style=wx.NO_BORDER)
        self.bitmap_button_delete.SetToolTip('Delete selected parameter set')
        self.bitmap_button_apply=wx.BitmapButton(self.tool_panel, -1
                                                 , wx.Bitmap(img.getstart_fitImage().Scale(tb_bmp_size, tb_bmp_size)),
                                                 size=size, style=wx.NO_BORDER)
        self.bitmap_button_apply.SetToolTip('Apply selected parameter set to the model')
        self.bitmap_button_plot=wx.BitmapButton(self.tool_panel, -1
                                                , wx.Bitmap(img.getplottingImage().Scale(tb_bmp_size, tb_bmp_size)),
                                                size=size, style=wx.NO_BORDER)
        self.bitmap_button_plot.SetToolTip('Toggle plotting of the selected parameter set')

        space=(2, -1)
        self.sizer_hor=wx.BoxSizer(wx.HORIZONTAL)
        self.sizer_hor.Add(self.bitmap_button_add, proportion=0, border=2)
        self.sizer_hor.Add(space)
        self.sizer_hor.Add(self.bitmap_button_delete, proportion=0, border=2)
        self.sizer_hor.Add(space)
        self.sizer_hor.Add(self.bitmap_button_apply, proportion=0, border=2)
        self.sizer_hor.Add(space)
        self.sizer_hor.Add(self.bitmap_button_plot, proportion=0, border=2)

        self.parameter_panel.Bind(wx.EVT_BUTTON, self.parameter_list.AddItem,
                                  self.bitmap_button_add)
        self.parameter_panel.Bind(wx.EVT_BUTTON, self.parameter_list.DeleteItem,
                                  self.bitmap_button_delete)
        self.parameter_panel.Bind(wx.EVT_BUTTON, self.parameter_list.ApplyParameters,
                                  self.bitmap_button_apply)
        self.parameter_panel.Bind(wx.EVT_BUTTON, self.parameter_list.TogglePlot,
                                  self.bitmap_button_plot)

    def create_parameter_list(self):
        '''
          Create a list of materials and it's graphical representation as
          well as a toolbar.
        '''
        self.tool_panel=wx.Panel(self.parameter_panel)
        self.parameter_list=ParameterList(self.parameter_panel, self)
        self.sizer_vert=wx.BoxSizer(wx.VERTICAL)
        self.parameter_panel.SetSizer(self.sizer_vert)

        self.create_toolbar()

        self.sizer_vert.Add(self.tool_panel, proportion=0, flag=wx.EXPAND, border=5)
        self.sizer_vert.Add((-1, 2))
        self.sizer_vert.Add(self.parameter_list, proportion=1, flag=wx.EXPAND, border=5)
        self.tool_panel.SetSizer(self.sizer_hor)

        ppanel=wx.Panel(self.parameter_panel)
        psizer=wx.GridSizer(2, 2, 0)
        ppanel.SetSizer(psizer)

        psizer.Add(wx.StaticText(ppanel, label='P1 index:'),
                   proportion=0, flag=wx.EXPAND, border=2)
        self.P1_index=wx.SpinCtrl(ppanel)
        self.P1_index.SetRange(0, 100)
        psizer.Add(self.P1_index, proportion=1, flag=wx.EXPAND, border=2)

        psizer.Add(wx.StaticText(ppanel, label='P2 index:'),
                   proportion=0, flag=wx.EXPAND, border=2)
        self.P2_index=wx.SpinCtrl(ppanel)
        self.P2_index.SetRange(0, 100)
        psizer.Add(self.P2_index, proportion=1, flag=wx.EXPAND, border=2)

        self.sizer_vert.Add(ppanel, proportion=0, flag=wx.EXPAND, border=5)

        self.parameter_panel.Bind(wx.EVT_SPINCTRL,
                                  self.parameter_list.UpdateParams, self.P1_index)
        self.parameter_panel.Bind(wx.EVT_SPINCTRL,
                                  self.parameter_list.UpdateParams, self.P2_index)

    def OnSimulate(self, event):
        '''OnSimulate(self, event) --> None
        
        Updates stuff after simulation
        '''
        colors=['b', 'r', 'g', 'c', 'm', 'y', 'k']
        styles=['--', '-.', ':']
        plot=self.parent.plot_data
        sld_plot=self.refplugin.sld_plot
        model=self.GetModel()
        k=0
        for ignore, do_plot, params, data, slds in self.parameter_list.parameter_list:
            if do_plot and params!=model.parameters.data:
                j=0
                for i, di in enumerate(data):
                    if model.data[i].show:
                        plot.ax.plot(di.x, di.y_sim, c=di.sim_color,
                                     lw=di.sim_linethickness, ls=styles[k%len(styles)],
                                     marker=di.sim_symbol, ms=di.sim_symbolsize)
                        for key in slds[i]:
                            is_imag=key[:2]=='Im' or key[:4]=='imag'
                            if (is_imag and self.refplugin.show_imag_sld) or not is_imag:
                                if key!='z' and key!='SLD unit':
                                    sld_plot.plot.ax.plot(slds[i]['z'], slds[i][key],
                                                          colors[j%len(colors)], ls=styles[k%len(styles)],
                                                          label=None)
                                    j+=1
                k+=1
        plot.AutoScale()
        plot.flush_plot()
        sld_plot.plot.flush_plot()

    def OnRecalcAll(self, event):
        pl=self.parameter_list
        model=self.GetModel()
        start_params=model.parameters.data
        for i in range(len(pl.parameter_list)):
            pl.Select(i, False)
        for i in range(len(pl.parameter_list)):
            pl.Select(i, True)
            if i>0:
                pl.Select(i-1, False)
            pl.ApplyParameters(None)
            model.simulate()
            slds=deepcopy(model.script_module.SLD)
            params=deepcopy(model.parameters.data)
            data=[di.copy() for di in model.data]
            pl.parameter_list[i]=[model.fom or 0., True, params, data, slds]
        pl.SetItemCount(len(pl.parameter_list))
        model.parameters.data=start_params
        self.OnAutoStore(None)

    def OnAutoStore(self, event):
        p=Parameters()
        base_name=self.GetModel().filename.rsplit('.', 1)[0]
        if self.mb_auto_store.IsChecked():
            for i, (ignore, ignore, params, ignore, ignore) in \
                    enumerate(self.parameter_list.parameter_list):
                p.data=params
                txt=p.get_ascii_output()
                open(base_name+'_%i.tab'%i, 'w').write(txt)

    def OnLoadAll(self, event):
        pl=self.parameter_list
        base_name=self.GetModel().filename.rsplit('.', 1)[0]
        if not os.path.exists(base_name+'_%i.tab'%0):
            return
        for ignore in range(len(pl.parameter_list)):
            pl.Select(0, True)
            pl.DeleteItem(None)
        i=0
        p=Parameters()
        model=self.GetModel()
        start_params=model.parameters.data
        while os.path.exists(base_name+'_%i.tab'%i):
            txt=open(base_name+'_%i.tab'%i, 'r').read()
            p.set_ascii_input(txt)
            model.parameters.data=deepcopy(p.data)
            model.simulate()
            pl.AddItem(None, simulate=False)
            i+=1
        model.parameters.data=start_params
        self.parent.eh_tb_simulate(None)

class ParameterList(wx.ListCtrl, ListCtrlAutoWidthMixin):
    '''
    The ListCtrl for the materials data.
    '''

    def __init__(self, parent, plugin):
        wx.ListCtrl.__init__(self, parent, -1,
                             style=wx.LC_REPORT | wx.LC_VIRTUAL | wx.LC_EDIT_LABELS)
        ListCtrlAutoWidthMixin.__init__(self)
        self.plugin=plugin
        dpi_scale_factor=wx.GetApp().dpi_scale_factor
        if sys.platform.startswith('win'):
            font=wx.Font(9,
                         wx.FONTFAMILY_SWISS, wx.FONTSTYLE_NORMAL,
                         wx.FONTWEIGHT_NORMAL, faceName="Lucida Sans Unicode")
        else:
            font=wx.Font(9,
                         wx.FONTFAMILY_ROMAN, wx.FONTSTYLE_NORMAL,
                         wx.FONTWEIGHT_NORMAL, encoding=wx.FONTENCODING_UNICODE)
        self.SetFont(font)

        # Set list length
        self.SetItemCount(0)

        self.parameter_list=[]
        # Set the column headers
        for col, (text, width) in enumerate([
            ("#", 24),
            ("FOM", 60),
            ("Show", 60),
            ("P1", 60),
            ("P2", 60),
            ]):
            self.InsertColumn(col, text, width=width)

    def OnSelectionChanged(self, evt):
        if not self.toggleshow:
            indices=self._GetSelectedItems()
            indices.sort()
            if not indices==self.show_indices:
                self.data_cont.show_data(indices)
                self._UpdateData('Show data set flag toggled',
                                 data_changed=True)
                # Forces update of list control
                self.SetItemCount(self.data_cont.get_count())
        evt.Skip()

    def OnGetItemText(self, item, col):
        if col==0:
            return str(item+1)
        data=self.parameter_list[item]
        if col==1:
            return "%.4g"%data[0]
        if col==2:
            return str(data[1])
        if col==3:
            try:
                idx=self.plugin.P1_index.GetValue()
                model=self.plugin.GetModel()
                idx=[i for i, p in enumerate(model.parameters.data) if p[2]][idx]
                return "%g"%data[2][idx][1]
            except:
                return ""
        if col==4:
            try:
                idx=self.plugin.P2_index.GetValue()
                model=self.plugin.GetModel()
                idx=[i for i, p in enumerate(model.parameters.data) if p[2]][idx]
                return "%g"%data[2][idx][1]
            except:
                return ""

    def _GetSelectedItems(self):
        ''' _GetSelectedItems(self) --> indices [list of integers]
        Function that yields a list of the currently selected items
        position in the list. In order of selction, i.e. no order.
        '''
        indices=[self.GetFirstSelected()]
        while indices[-1]!=-1:
            indices.append(self.GetNextSelected(indices[-1]))

        # Remove the last will be -1
        indices.pop(-1)
        return indices

    def _CheckSelected(self, indices):
        '''_CheckSelected(self, indices) --> bool
        Checks so at least data sets are selcted, otherwise show a dialog box
        and return False
        '''
        # Check so that one dataset is selected
        if len(indices)==0:
            dlg=wx.MessageDialog(self,
                                 'At least one data set has to be selected'
                                 , caption='Information', style=wx.OK | wx.ICON_INFORMATION)
            dlg.ShowModal()
            dlg.Destroy()
            return False
        return True

    def DeleteItem(self, ignore):
        index=self.GetFirstSelected()
        self.parameter_list.pop(index)
        self.SetItemCount(len(self.parameter_list))
        self.plugin.OnAutoStore(None)

    def AddItem(self, ignore, simulate=True):
        model=self.plugin.GetModel()
        slds=deepcopy(model.script_module.SLD)
        params=deepcopy(model.parameters.data)
        data=[di.copy() for di in model.data]
        self.parameter_list.append([model.fom or 0., True, params, data, slds])
        self.SetItemCount(len(self.parameter_list))
        self.plugin.OnAutoStore(None)
        if simulate:
            self.plugin.parent.eh_tb_simulate(None)

    def ApplyParameters(self, ignore):
        index=self.GetFirstSelected()
        model=self.plugin.GetModel()
        was_stored=False
        for ignore, ignore, params, ignore, ignore in self.parameter_list:
            if model.parameters.data==params:
                was_stored=True
                break
        if not was_stored:
            message='Do you want to store the current parameters in the Vault'+ \
                    ' before applying the selected ones?'
            dlg=wx.MessageDialog(self, message, 'Store current parameters?',
                                 wx.YES_NO | wx.ICON_QUESTION)
            if dlg.ShowModal()==wx.ID_YES:
                self.AddItem(None)
        model.parameters.data=deepcopy(self.parameter_list[index][2])
        self.plugin.parent.paramter_grid._grid_changed()
        self.plugin.parent.paramter_grid.grid.ForceRefresh()
        self.plugin.parent.eh_tb_simulate(None)

    def TogglePlot(self, ignore):
        index=self.GetFirstSelected()
        self.parameter_list[index][1]=not self.parameter_list[index][1]
        self.RefreshItem(index)
        self.plugin.parent.eh_tb_simulate(None)

    def UpdateParams(self, ignore):
        if len(self.parameter_list)==0:
            return
        self.RefreshItems(0, len(self.parameter_list)-1)



"""
Library defining the standard colours that should be used in GenX. All colours are based on those defined
in the TANGO desktop project, http://tango.freedesktop.org
"""

__author__='Matts Bjorck'

import wx

HIGHLIGHT=0
NORMAL=1
SHADOW=2

class StandardColours:
    highlight=0
    normal=1
    shadow=2
    butter=((0xfc, 0xe9, 0x4f), (0xed, 0xd4, 0x00), (0xc4, 0xa0, 0x00))
    aluminum=((0xee, 0xee, 0xec), (0xd3, 0xd7, 0xcf), (0xba, 0xbd, 0xb6))
    chameleon=((0x8a, 0xe2, 0x34), (0x73, 0xd2, 0x16), (0x4e, 0x9a, 0x06))
    orange=((0xfc, 0xaf, 0x3e), (0xf5, 0x79, 0x00), (0xce, 0x5c, 0x00))
    chocolate=((0xe9, 0xb9, 0x6e), (0xc1, 0x7d, 0x11), (0x8f, 0x59, 0x02))
    sky_blue=((0x72, 0x9f, 0xcf), (0x34, 0x65, 0xa4), (0x20, 0x4a, 0x87))
    plum=((0xad, 0x7f, 0xa8), (0x75, 0x50, 0x7b), (0x5c, 0x35, 0x66))
    slate=((0x88, 0x8a, 0x85), (0x55, 0x57, 0x53), (0x2e, 0x34, 0x36))
    scarlet_red=((0xef, 0x29, 0x29), (0xcc, 0x00, 0x00), (0xa4, 0x00, 0x00))

    translations={'red': 'scarlet_red', 'blue': 'sky_blue', 'green': 'chameleon', 'yellow': 'butter'}

    def get_rgb(self, name, appearance=NORMAL):
        """ Return a colour from the TANGO palette.

        :param name: The name of the colour
        :param appearance: One of the values HIGHLIGHT, NORMAL or SHADOW, default is NORMAL
        :return: A tuple representing the RGB colour
        """

        return getattr(self, self._translate_name(name))[appearance]

    def get_colour(self, name, appearance=NORMAL):
        """ Return the wx.Colour object for named colour.

        :param name:  The name of the colour
        :param appearance: One of the values HIGHLIGHT, NORMAL or SHADOW, default is NORMAL
        :return: wx.Colour object
        """
        rgb=self.get_rgb(name, appearance)
        return wx.Colour(rgb[0], rgb[1], rgb[2])

    def _translate_name(self, name):
        if name.lower() in self.translations:
            return self.translations[name.lower()]
        else:
            return name.lower()

colours=StandardColours()



# File: plotpanel.py a extension of panel that implements matplotlib plotting
# libary.
# Programmed by: Matts Bjorck
# Last changed: 2009 03 10

import warnings
import matplotlib
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg
from matplotlib.backends.backend_agg import RendererAgg
from matplotlib.figure import Figure

from numpy import *
import wx
import wx.lib.newevent
from wx import PAPER_A4, LANDSCAPE

from . import filehandling as io
from .gui_logging import iprint
from logging import debug, getLogger, ERROR

# deactivate matplotlib logging that we are not interested in
getLogger('matplotlib.ticker').setLevel(ERROR)
getLogger('matplotlib.font_manager').setLevel(ERROR)

# Event for a click inside an plot which yields a number
(plot_position, EVT_PLOT_POSITION)=wx.lib.newevent.NewEvent()
# Event to tell the main window that the zoom state has changed
(state_changed, EVT_PLOT_SETTINGS_CHANGE)=wx.lib.newevent.NewEvent()

zoom_state=False

# ==============================================================================
class PlotPanel(wx.Panel):
    ''' Base class for the plotting in GenX - all the basic functionallity
        should be implemented in this class. The plots should be derived from
        this class. These classes should implement an update method to update
        the plots. 
    '''

    def __init__(self, parent, id=-1, color=None, dpi=None
                 , style=wx.NO_FULL_REPAINT_ON_RESIZE | wx.EXPAND | wx.ALL
                 , config=None, config_name='', **kwargs):
        debug('start init PlotPanel')
        wx.Panel.__init__(self, parent, id=id, style=style, **kwargs)
        if dpi is None:
            dpi=wx.GetApp().dpi_scale_factor*96.  # wx.GetDisplayPPI()[0]
        self.parent=parent
        self.callback_window=self
        self.config=config
        self.config_name=config_name
        debug('init PlotPanel - setup figure')
        self.figure=Figure(figsize=(1.0, 1.0), dpi=dpi)
        debug('init PlotPanel - setup canvas')
        self.canvas=FigureCanvasWxAgg(self, -1, self.figure)
        self.canvas.SetExtraStyle(wx.EXPAND)
        self.SetColor(color)
        self._resizeflag=True
        self.print_size=(15./2.54, 12./2.54)
        # self._SetSize()

        # Flags and bindings for zooming
        self.zoom=False
        self.zooming=False
        self.scale='linear'
        self.autoscale=True

        debug('init PlotPanel - bind events')
        self.Bind(wx.EVT_IDLE, self._onIdle)
        self.Bind(wx.EVT_SIZE, self._onSize)
        self.canvas.Bind(wx.EVT_LEFT_DOWN, self.OnLeftMouseButtonDown)
        self.canvas.Bind(wx.EVT_LEFT_UP, self.OnLeftMouseButtonUp)
        self.canvas.Bind(wx.EVT_MOTION, self.OnMouseMove)
        self.canvas.Bind(wx.EVT_LEFT_DCLICK, self.OnLeftDblClick)
        self.canvas.Bind(wx.EVT_RIGHT_UP, self.OnContextMenu)

        cursor=wx.Cursor(wx.CURSOR_CROSS)
        self.canvas.SetCursor(cursor)
        self.old_scale_state=True
        self.ax=None

        debug('init PlotPanel - FigurePrinter and Bitmap')
        # Init printout stuff
        self.fig_printer=FigurePrinter(self)

        # Create the drawing bitmap
        self.bitmap=wx.Bitmap(1, 1, depth=wx.BITMAP_SCREEN_DEPTH)
        #        DEBUG_MSG("__init__() - bitmap w:%d h:%d" % (w,h), 2, self)
        self._isDrawn=False
        debug('end init PlotPanel')

    def SetColor(self, rgbtuple=None):
        ''' Set the figure and canvas color to be the same '''
        if not rgbtuple:
            rgbtuple=self.parent.GetBackgroundColour()
            # wx.SystemSettings.GetColour(wx.SYS_COLOUR_BTNFACE).Get()
        col=[c/255. for c in rgbtuple]
        self.figure.set_facecolor(col)
        self.figure.set_edgecolor(col)
        self.canvas.SetBackgroundColour(wx.Colour(*rgbtuple))

    def _onSize(self, evt):
        self._resizeflag=True
        self._SetSize()
        # self.canvas.draw(repaint = False)

    def _onIdle(self, evt):
        if self._resizeflag:
            self._resizeflag=False
            self._SetSize()
            # self.canvas.gui_repaint(drawDC = wx.PaintDC(self))

    def _SetSize(self, pixels=None):
        ''' This method can be called to force the Plot to be a desired 
            size which defaults to the ClientSize of the Panel.
        '''
        if not pixels:
            pixels=self.GetClientSize()
        if pixels[0]==0 or pixels[1]==0:
            return

        self.canvas.SetSize(pixels)
        with warnings.catch_warnings():
            warnings.simplefilter("ignore", UserWarning)
            try:
                self.figure.tight_layout(h_pad=0)
            except ValueError:
                pass
        # self.figure.set_size_inches(pixels[0]/self.figure.get_dpi()
        # , pixels[1]/self.figure.get_dpi())

    def ReadConfig(self):
        '''ReadConfig(self) --> None
        
        Reads in the config file
        '''
        bool_items=['zoom', 'autoscale']
        bool_func=[self.SetZoom, self.SetAutoScale]

        if not self.config:
            return

        vals=[]
        for index in range(len(bool_items)):
            try:
                val=self.config.get_boolean(self.config_name,
                                            bool_items[index])
            except io.OptionError as e:
                iprint('Could not locate option %s.%s' \
                       %(self.config_name, bool_items[index]))
                vals.append(None)
            else:
                vals.append(val)

        try:
            scale=self.config.get(self.config_name, 'y scale')
        except io.OptionError as e:
            iprint('Could not locate option %s.%s' \
                   %(self.config_name, 'y scale'))
        else:
            self.SetYScale(scale)

        try:
            scale=self.config.get(self.config_name, 'x scale')
        except io.OptionError as e:
            iprint('Could not locate option %s.%s' \
                   %(self.config_name, 'x scale'))
            self.SetXScale('lin')
        else:
            self.SetXScale(scale)

        # This is done due to that the zoom and autoscale has to read 
        # before any commands are issued in order not to overwrite 
        # the config
        [bool_func[i](vals[i]) for i in range(len(vals)) if vals[i]]

    def WriteConfig(self, zoom=None, autoscale=None, yscale=None, xscale=None):
        '''WriteConfig(self) --> None
        
        Writes the current settings to the config file
        '''
        if self.config:
            if zoom is not None:
                self.config.set(self.config_name, 'zoom', zoom)
            if autoscale is not None:
                self.config.set(self.config_name, 'autoscale', autoscale)
            if yscale is not None:
                self.config.set(self.config_name, 'y scale', yscale)
            if xscale is not None:
                self.config.set(self.config_name, 'x scale', xscale)

    def SetZoom(self, active=False):
        '''
        set the zoomstate
        '''
        # if not self.zoom_sel:
        # self.zoom_sel = RectangleSelector(self.ax,\
        # self.box_select_callback, drawtype='box',useblit=False)
        # print help(self.zoom_sel.ignore)

        if active:
            # self.zoom_sel.ignore = lambda x: False
            self.zoom=True
            cursor=wx.Cursor(wx.CURSOR_MAGNIFIER)
            self.canvas.SetCursor(cursor)
            if self.callback_window:
                evt=state_changed(zoomstate=True, yscale=self.GetYScale(), autoscale=self.autoscale,
                                  xscale=self.GetXScale())
                wx.PostEvent(self.callback_window, evt)
            if self.ax:
                # self.ax.set_autoscale_on(False)
                self.old_scale_state=self.GetAutoScale()
                self.SetAutoScale(False)

        else:
            # self.zoom_sel.ignore = lambda x: True
            self.zoom=False
            cursor=wx.Cursor(wx.CURSOR_CROSS)
            self.canvas.SetCursor(cursor)
            if self.callback_window:
                evt=state_changed(zoomstate=False, yscale=self.GetYScale(), autoscale=self.autoscale,
                                  xscale=self.GetXScale())
                wx.PostEvent(self.callback_window, evt)
            if self.ax:
                # self.ax.set_autoscale_on(self.autoscale)
                self.SetAutoScale(self.old_scale_state)
        self.WriteConfig(zoom=self.zoom)

    def GetZoom(self):
        '''GetZoom(self) --> state [bool]
        Returns the zoom state of the plot panel. 
        True = zoom active
        False = zoom inactive
        '''
        return self.zoom

    def SetAutoScale(self, state):
        '''SetAutoScale(self, state) --> None
        
        Sets autoscale of the main axes wheter or not it should autoscale
        when plotting
        '''
        # self.ax.set_autoscale_on(state)
        self.autoscale=state
        self.WriteConfig(autoscale=self.autoscale)
        evt=state_changed(zoomstate=self.GetZoom(),
                          yscale=self.GetYScale(), autoscale=self.autoscale,
                          xscale=self.GetXScale())
        wx.PostEvent(self.callback_window, evt)

    def GetAutoScale(self):
        '''GetAutoScale(self) --> state [bool]
        
        Returns the autoscale state, true if the plots is automatically
        scaled for each plot command.
        '''
        return self.autoscale

    def AutoScale(self, force=False):
        '''AutoScale(self) --> None
        
        A log safe way to autoscale the plots - the ordinary axis tight 
        does not work for negative log data. This works!
        '''

        if not (self.autoscale or force):
            return
        # If nothing is plotted no autoscale use defaults...
        if sum([len(line.get_ydata())>0 for line in self.ax.lines])==0:
            self.ax.set_xlim(0, 1)
            self.ax.set_ylim(1e-3, 1.0)
            return

        if self.scale=='log':
            # Find the lowest possible value of all the y-values that are
            # greater than zero. check so that y data contain data before min
            # is applied
            tmp=[line.get_ydata().compress(line.get_ydata()>0.0).min() \
                 for line in self.ax.lines if array(line.get_ydata()>0.0).sum()>0]
            if len(tmp)>0:
                ymin=min(tmp)
            else:
                ymin=1e-3
            tmp=[line.get_ydata().compress(line.get_ydata()>0.0).max() \
                 for line in self.ax.lines if array(line.get_ydata()>0.0).sum()>0]
            if len(tmp)>0:
                ymax=max(tmp)
            else:
                ymax=1
        else:
            ymin=min([array(line.get_ydata()).compress(isfinite(line.get_ydata())).min()
                      for line in self.ax.lines if len(line.get_ydata())>0 and any(isfinite(line.get_ydata()))])
            ymax=max([array(line.get_ydata()).compress(isfinite(line.get_ydata())).max()
                      for line in self.ax.lines if len(line.get_ydata())>0 and any(isfinite(line.get_ydata()))])
        tmp=[array(line.get_xdata()).min() for line in self.ax.lines if len(line.get_ydata())>0]
        if len(tmp)>0:
            xmin=min(tmp)
        else:
            xmin=0
        tmp=[array(line.get_xdata()).max() \
             for line in self.ax.lines if len(line.get_ydata())>0]
        if len(tmp)>0:
            xmax=max(tmp)
        else:
            xmax=1
        # Set the limits
        try:
            if xmin!=xmax:
                self.ax.set_xlim(xmin, xmax)
            if ymin!=ymax:
                self.ax.set_ylim(ymin*(1-sign(ymin)*0.05), ymax*(1+sign(ymax)*0.05))
            self.flush_plot()
        except UserWarning:
            pass

    def SetYScale(self, scalestring):
        ''' SetYScale(self, scalestring) --> None
        
        Sets the y-scale of the main plotting axes. Currently accepts
        'log' or 'lin'.
        '''
        if not self.ax:
            return
        if scalestring=='log':
            self.scale='log'
        elif scalestring=='linear' or scalestring=='lin':
            self.scale='linear'
        else:
            raise ValueError('Not allowed scaling')

        # do nothing if no data in current plot
        self.AutoScale(force=True)

        try:
            self.ax.set_yscale(self.scale)
        except OverflowError:
            self.AutoScale(force=True)
        except UserWarning:
            pass

        try:
            self.flush_plot()
        except UserWarning:
            pass
        self.WriteConfig(yscale=self.scale)
        evt=state_changed(zoomstate=self.GetZoom(),
                          yscale=self.GetYScale(), autoscale=self.autoscale,
                          xscale=self.GetXScale())
        wx.PostEvent(self.callback_window, evt)

    def SetXScale(self, scalestring):
        ''' SetXScale(self, scalestring) --> None

        Sets the x-scale of the main plotting axes. Currently accepts
        'log' or 'lin'.
        '''
        if not self.ax:
            return
        if scalestring=='log':
            self.x_scale='log'
            self.AutoScale(force=True)
            try:
                self.ax.set_xscale('log')
            except OverflowError:
                self.AutoScale(force=True)
            except UserWarning:
                pass
        elif scalestring=='linear' or scalestring=='lin':
            self.x_scale='linear'
            self.ax.set_xscale('linear')
            self.AutoScale(force=True)
        else:
            raise ValueError('Not allowed scaling')
        try:
            self.flush_plot()
        except UserWarning:
            pass
        self.WriteConfig(xscale=self.x_scale)
        evt=state_changed(zoomstate=self.GetZoom(),
                          yscale=self.GetYScale(), autoscale=self.autoscale,
                          xscale=self.GetXScale())
        wx.PostEvent(self.callback_window, evt)

    def GetYScale(self):
        '''GetYScale(self) --> String
        
        Returns the current y-scale in use. Currently the string
        'log' or 'linear'. If the axes does not exist it returns None.
        '''
        if self.ax:
            return self.ax.get_yscale()
        else:
            return None

    def GetXScale(self):
        '''GetXScale(self) --> String

        Returns the current x-scale in use. Currently the string
        'log' or 'linear'. If the axes does not exist it returns None.
        '''
        if self.ax:
            return self.ax.get_xscale()
        else:
            return None

    def CopyToClipboard(self, event=None):
        '''CopyToClipboard(self, event) --> None
        
        Copy the plot to the clipboard.
        '''
        self.SetColor((255, 255, 255))
        self.canvas.draw()
        self.canvas.Copy_to_Clipboard(event=event)
        self.SetColor()
        self.canvas.draw()

    def PrintSetup(self, event=None):
        '''PrintSetup(self) --> None
        
        Sets up the printer. Creates a dialog box
        '''
        self.fig_printer.pageSetup()

    def PrintPreview(self, event=None):
        '''PrintPreview(self) --> None
        
        Prints a preview on screen.
        '''
        self.fig_printer.previewFigure(self.figure)

    def Print(self, event=None):
        '''Print(self) --> None
        
        Print the figure.
        '''
        self.fig_printer.printFigure(self.figure)

    def SetCallbackWindow(self, window):
        '''SetCallbackWindow(self, window) --> None
        
        Sets the callback window that should recieve the events from 
        picking.
        '''

        self.callback_window=window

    def OnLeftDblClick(self, event):
        if self.ax and self.zoom:
            tmp=self.GetAutoScale()
            self.SetAutoScale(True)
            self.AutoScale()
            self.SetAutoScale(tmp)
            # self.AutoScale()
            # self.flush_plot()
            # self.ax.set_autoscale_on(False)

    def OnLeftMouseButtonDown(self, event):
        self.start_pos=event.Position

        # print 'Left Mouse button pressed ', self.ax.transData.inverse_xy_tup(self.start_pos)
        class Point:
            pass

        p=Point()
        p.x, p.y=self.start_pos
        size=self.canvas.GetClientSize()
        p.y=(size.height-p.y)
        if self.zoom and self.ax:
            in_axes=self.ax.in_axes(p)
            if in_axes:
                self.zooming=True
                self.cur_rect=None
                self.canvas.CaptureMouse()
                self.overlay=wx.Overlay()
            else:
                self.zooming=False
        elif self.ax:
            size=self.canvas.GetClientSize()
            xy=self.ax.transData.inverted().transform(
                array([self.start_pos[0], size.height-self.start_pos[1]])
                [newaxis, :])
            x, y=xy[0, 0], xy[0, 1]
            if self.callback_window:
                evt=plot_position(text='(%.3e, %.3e)'%(x, y))
                wx.PostEvent(self.callback_window, evt)
            # print x,y

    def OnMouseMove(self, event):
        if self.zooming and event.Dragging() and event.LeftIsDown():
            self.cur_pos=event.Position

            # print 'Mouse Move ', self.ax.transData.inverse_xy_tup(self.cur_pos)
            class Point:
                pass

            p=Point()
            p.x, p.y=self.cur_pos
            size=self.canvas.GetClientSize()
            p.y=(size.height-p.y)
            in_axes=self.ax.in_axes(p)
            if in_axes:
                new_rect=(min(self.start_pos[0], self.cur_pos[0]),
                          min(self.start_pos[1], self.cur_pos[1]),
                          abs(self.cur_pos[0]-self.start_pos[0]),
                          abs(self.cur_pos[1]-self.start_pos[1]))
                self._DrawAndErase(new_rect, self.cur_rect)
                self.cur_rect=new_rect
        # event.Skip()

    def OnLeftMouseButtonUp(self, event):
        if self.canvas.HasCapture():
            # print 'Left Mouse button up'
            self.canvas.ReleaseMouse()
            del self.overlay
            if self.zooming and self.cur_rect:
                # Note: The coordinte system for matplotlib have a different 
                # direction of the y-axis and a different origin!
                size=self.canvas.GetClientSize()
                start=self.ax.transData.inverted().transform(
                    array([self.start_pos[0], size.height-self.start_pos[1]])[newaxis, :])
                end=self.ax.transData.inverted().transform(
                    array([self.cur_pos[0], size.height-self.cur_pos[1]])[newaxis, :])
                xend, yend=end[0, 0], end[0, 1]
                xstart, ystart=start[0, 0], start[0, 1]

                # print xstart, xend
                # print ystart, yend
                self.ax.set_xlim(min(xstart, xend), max(xstart, xend))
                self.ax.set_ylim(min(ystart, yend), max(ystart, yend))
                self.flush_plot()
            self.zooming=False

    def _DrawAndErase(self, box_to_draw, box_to_erase=None):
        '''_DrawAndErase(self, box_to_draw, box_to_erase = None) --> None
        '''
        rect=wx.Rect(*box_to_draw)

        # Draw the rubber-band rectangle using an overlay so it
        # will manage keeping the rectangle and the former window
        # contents separate.
        # dc = wx.ClientDC(self)
        dc=wx.ClientDC(self.canvas)
        odc=wx.DCOverlay(self.overlay, dc)
        odc.Clear()

        dc.SetPen(wx.Pen("black", 2, style=wx.DOT_DASH))
        dc.SetBrush(wx.Brush("black", style=wx.BRUSHSTYLE_TRANSPARENT))
        dc.DrawRectangle(rect)

    def OnContextMenu(self, event):
        '''OnContextMenu(self, event) --> None
        
        Callback to show the popmenu for the plot which allows various 
        settings to be made.
        '''
        menu=wx.Menu()

        zoomID=wx.NewId()
        menu.AppendCheckItem(zoomID, "Zoom")
        menu.Check(zoomID, self.GetZoom())

        def OnZoom(event):
            self.SetZoom(not self.GetZoom())

        self.Bind(wx.EVT_MENU, OnZoom, id=zoomID)

        zoomallID=wx.NewId()
        menu.Append(zoomallID, 'Zoom All')

        def zoomall(event):
            tmp=self.GetAutoScale()
            self.SetAutoScale(True)
            self.AutoScale()
            self.SetAutoScale(tmp)
            # self.flush_plot()

        self.Bind(wx.EVT_MENU, zoomall, id=zoomallID)

        copyID=wx.NewId()
        menu.Append(copyID, "Copy")

        def copy(event):
            self.CopyToClipboard()

        self.Bind(wx.EVT_MENU, copy, id=copyID)

        yscalemenu=wx.Menu()
        logID=wx.NewId()
        linID=wx.NewId()
        yscalemenu.AppendRadioItem(logID, "log")
        yscalemenu.AppendRadioItem(linID, "linear")
        menu.Append(-1, "y-scale", yscalemenu)
        if self.GetYScale()=='log':
            yscalemenu.Check(logID, True)
        else:
            yscalemenu.Check(linID, True)

        def yscale_log(event):
            if self.ax:
                self.SetYScale('log')
                self.AutoScale()
                self.flush_plot()

        def yscale_lin(event):
            if self.ax:
                self.SetYScale('lin')
                self.AutoScale()
                self.flush_plot()

        self.Bind(wx.EVT_MENU, yscale_log, id=logID)
        self.Bind(wx.EVT_MENU, yscale_lin, id=linID)

        xscalemenu=wx.Menu()
        logID=wx.NewId()
        linID=wx.NewId()
        xscalemenu.AppendRadioItem(logID, "log")
        xscalemenu.AppendRadioItem(linID, "linear")
        menu.Append(-1, "x-scale", xscalemenu)
        if self.GetXScale()=='log':
            xscalemenu.Check(logID, True)
        else:
            xscalemenu.Check(linID, True)

        def xscale_log(event):
            if self.ax:
                self.SetXScale('log')
                self.AutoScale()
                self.flush_plot()

        def xscale_lin(event):
            if self.ax:
                self.SetXScale('lin')
                self.AutoScale()
                self.flush_plot()

        self.Bind(wx.EVT_MENU, xscale_log, id=logID)
        self.Bind(wx.EVT_MENU, xscale_lin, id=linID)

        autoscaleID=wx.NewId()
        menu.AppendCheckItem(autoscaleID, "Autoscale")
        menu.Check(autoscaleID, self.GetAutoScale())

        def OnAutoScale(event):
            self.SetAutoScale(not self.GetAutoScale())

        self.Bind(wx.EVT_MENU, OnAutoScale, id=autoscaleID)

        # Time to show the menu
        self.PopupMenu(menu)

        self.Unbind(wx.EVT_MENU)
        menu.Destroy()

    def flush_plot(self):
        # self._SetSize()
        # self.canvas.gui_repaint(drawDC = wx.PaintDC(self))
        # self.ax.set_yscale(self.scale)
        with warnings.catch_warnings():
            warnings.simplefilter("ignore", UserWarning)
            self.figure.tight_layout(h_pad=0)
        self.canvas.draw()

    def update(self, data):
        pass

# ==============================================================================
# Print out class borrowed from wxmpl
if callable(getattr(wx, 'PostScriptDC_SetResolution', None)):
    wx.PostScriptDC_SetResolution(300)

class FigurePrinter:
    """
    Provides a simplified interface to the wxPython printing framework that's
    designed for printing matplotlib figures.
    """

    def __init__(self, view, printData=None):
        """
        Create a new C{FigurePrinter} associated with the wxPython widget
        C{view}.  The keyword argument C{printData} supplies a C{wx.PrintData}
        object containing the default printer settings.
        """
        self.view=view

        if printData is None:
            self.pData=wx.PrintData()
        else:
            self.pData=printData

        self.pData.SetPaperId(PAPER_A4)
        self.pData.SetOrientation(LANDSCAPE)
        self.pData.SetNoCopies(1)

    def destroy(self):
        """
        Sets this object's C{view} attribute to C{None}.
        """
        self.view=None

    def getPrintData(self):
        """
        Return the current printer settings in their C{wx.PrintData} object.
        """
        return self.pData

    def setPrintData(self, printData):
        """
        Use the printer settings in C{printData}.
        """
        self.pData=printData

    def pageSetup(self):
        dlg=wx.PrintDialog(self.view)
        pdData=dlg.GetPrintDialogData()
        pdData.SetPrintData(self.pData)
        # pdData.SetSetupDialog(True)

        if dlg.ShowModal()==wx.ID_OK:
            self.pData=pdData.GetPrintData()
        dlg.Destroy()
        self.copyPrintData()

    def previewFigure(self, figure, title=None):
        """
        Open a "Print Preview" window for the matplotlib chart C{figure}.  The
        keyword argument C{title} provides the printing framework with a title
        for the print job.
        """
        window=self.view
        while not isinstance(window, wx.Frame):
            window=window.GetParent()
            assert window is not None

        fpo=FigurePrintout(figure, title)
        fpo4p=FigurePrintout(figure, title)
        preview=wx.PrintPreview(fpo, fpo4p, self.pData)
        frame=wx.PreviewFrame(preview, window, 'Print Preview')
        if self.pData.GetOrientation()==wx.PORTRAIT:
            frame.SetSize(wx.Size(450, 625))
        else:
            frame.SetSize(wx.Size(600, 500))
        frame.Initialize()
        frame.Show(True)
        self.copyPrintData()

    def printFigure(self, figure, title=None):
        """
        Open a "Print" dialog to print the matplotlib chart C{figure}.  The
        keyword argument C{title} provides the printing framework with a title
        for the print job.
        """
        pdData=wx.PrintDialogData()
        pdData.SetFromPage(1)
        pdData.SetToPage(1)
        pdData.SetPrintToFile(True)
        pdData.SetPrintData(self.pData)
        printer=wx.Printer(pdData)
        fpo=FigurePrintout(figure, title)
        fpo.SetPPIPrinter(300, 300)
        self.pData=pdData.GetPrintData()
        if printer.Print(self.view, fpo, True):
            self.pData=pdData.GetPrintData()
        self.copyPrintData()

    def copyPrintData(self):
        '''Create a copy of the print data to avoid seg faults
        '''
        pData_new=wx.PrintData()
        pData_new.SetBin(self.pData.GetBin())
        pData_new.SetCollate(self.pData.GetCollate())
        pData_new.SetColour(self.pData.GetColour())
        pData_new.SetDuplex(self.pData.GetDuplex())
        pData_new.SetFilename(self.pData.GetFilename())
        pData_new.SetNoCopies(self.pData.GetNoCopies())
        pData_new.SetOrientation(self.pData.GetOrientation())
        pData_new.SetPaperId(self.pData.GetPaperId())
        pData_new.SetPaperSize(self.pData.GetPaperSize())
        pData_new.SetPrinterName(self.pData.GetPrinterName())
        pData_new.SetPrintMode(self.pData.GetPrintMode())
        pData_new.SetPrivData(self.pData.GetPrivData())
        pData_new.SetQuality(self.pData.GetQuality())
        self.pData=pData_new

class FigurePrintout(wx.Printout):
    """
    Render a matplotlib C{Figure} to a page or file using wxPython's printing
    framework.
    """

    ASPECT_RECTANGULAR=1
    ASPECT_SQUARE=2

    def __init__(self, figure, title=None, size=None, aspectRatio=None):
        """
        Create a printout for the matplotlib chart C{figure}.  The
        keyword argument C{title} provides the printing framework with a title
        for the print job.  The keyword argument C{size} specifies how to scale
        the figure, from 1 to 100 percent.  The keyword argument C{aspectRatio}
        determines whether the printed figure will be rectangular or square.
        """
        self.figure=figure

        figTitle=figure.gca().title.get_text()
        if not figTitle:
            figTitle=title or 'GenX plot'

        if size is None:
            size=100
        elif size<0 or size>100:
            raise ValueError('invalid figure size')
        self.size=size

        if aspectRatio is None:
            aspectRatio=self.ASPECT_RECTANGULAR
        elif (aspectRatio!=self.ASPECT_RECTANGULAR
              and aspectRatio!=self.ASPECT_SQUARE):
            raise ValueError('invalid aspect ratio')
        self.aspectRatio=aspectRatio

        wx.Printout.__init__(self, figTitle)
        # self.SetPPIPrinter(300, 300)

    def GetPageInfo(self):
        """
        Overrides wx.Printout.GetPageInfo() to provide the printing framework
        with the number of pages in this print job.
        """
        return 1, 1, 1, 1

    def OnPrintPage(self, pageNumber):
        """
        Overrides wx.Printout.OnPrintPage to render the matplotlib figure to
        a printing device context.
        """
        # % of printable area to use
        imgPercent=max(1, min(100, self.size))/100.0

        # ratio of the figure's width to its height
        if self.aspectRatio==self.ASPECT_RECTANGULAR:
            aspectRatio=1.61803399
        elif self.aspectRatio==self.ASPECT_SQUARE:
            aspectRatio=1.0
        else:
            raise ValueError('invalid aspect ratio')

        # Device context to draw the page
        dc=self.GetDC()

        # PPI_P: Pixels Per Inch of the Printer
        wPPI_P, hPPI_P=[float(x) for x in self.GetPPIPrinter()]
        PPI_P=(wPPI_P+hPPI_P)/2.0

        # PPI: Pixels Per Inch of the DC
        if self.IsPreview():
            wPPI, hPPI=[float(x) for x in self.GetPPIScreen()]
        else:
            wPPI, hPPI=wPPI_P, hPPI_P
        PPI=(wPPI+hPPI)/2.0

        # Pg_Px: Size of the page (pixels)
        wPg_Px, hPg_Px=[float(x) for x in self.GetPageSizePixels()]

        # Dev_Px: Size of the DC (pixels)
        wDev_Px, hDev_Px=[float(x) for x in self.GetDC().GetSize()]

        # Pg: Size of the page (inches)
        wPg=wPg_Px/PPI_P
        hPg=hPg_Px/PPI_P

        # minimum margins (inches)
        # TODO: make these arguments to __init__()
        wM=0.75
        hM=0.75

        # Area: printable area within the margins (inches)
        wArea=wPg-2*wM
        hArea=hPg-2*hM

        # Fig: printing size of the figure
        # hFig is at a maximum when wFig == wArea
        max_hFig=wArea/aspectRatio
        hFig=min(imgPercent*hArea, max_hFig)
        wFig=aspectRatio*hFig

        # scale factor = device size / page size (equals 1.0 for real printing)
        S=((wDev_Px/PPI)/wPg+(hDev_Px/PPI)/hPg)/2.0

        # Fig_S: scaled printing size of the figure (inches)
        # M_S: scaled minimum margins (inches)
        wFig_S=S*wFig
        hFig_S=S*hFig
        wM_S=S*wM
        hM_S=S*hM

        # Fig_Dx: scaled printing size of the figure (device pixels)
        # M_Dx: scaled minimum margins (device pixels)
        wFig_Dx=int(S*PPI*wFig)
        hFig_Dx=int(S*PPI*hFig)
        wM_Dx=int(S*PPI*wM)
        hM_Dx=int(S*PPI*hM)

        image=self.render_figure_as_image(wFig, hFig, PPI)

        if self.IsPreview():
            image=image.Scale(wFig_Dx, hFig_Dx)
        self.GetDC().DrawBitmap(image.ConvertToBitmap(), wM_Dx, hM_Dx, False)

        return True

    def render_figure_as_image(self, wFig, hFig, dpi):
        """
        Renders a matplotlib figure using the Agg backend and stores the result
        in a C{wx.Image}.  The arguments C{wFig} and {hFig} are the width and
        height of the figure, and C{dpi} is the dots-per-inch to render at.
        """
        figure=self.figure

        old_dpi=figure.dpi.get()
        figure.dpi.set(dpi)
        old_width=figure.figwidth.get()
        figure.figwidth.set(wFig)
        old_height=figure.figheight.get()
        figure.figheight.set(hFig)

        wFig_Px=int(figure.bbox.width())
        hFig_Px=int(figure.bbox.height())

        agg=RendererAgg(wFig_Px, hFig_Px, dpi)

        figure.draw(agg)

        figure.dpi.set(old_dpi)
        figure.figwidth.set(old_width)
        figure.figheight.set(old_height)

        image=wx.EmptyImage(wFig_Px, hFig_Px)
        image.SetData(agg.tostring_rgb())
        return image

# ==============================================================================
class DataPlotPanel(PlotPanel):
    ''' Class for plotting the data and the fit
    '''
    _last_poptions=None

    def __init__(self, parent, id=-1, color=None, dpi=None
                 , style=wx.NO_FULL_REPAINT_ON_RESIZE, **kwargs):
        self.main_ax_rect=(0.125, 0.3, 0.8, 0.6)
        self.sub_ax_rect=(0.125, 0.1, 0.8, 0.18)
        PlotPanel.__init__(self, parent, id, color, dpi, style, **kwargs)
        self.create_axes()
        self.update=self.plot_data
        self.SetAutoScale(True)
        # self.ax = self.figure.add_subplot(111)
        # self.ax.set_autoscale_on(False)

    def SetXScale(self, scalestring):
        ''' SetXScale(self, scalestring) --> None

        Sets the x-scale of the main plotting axes. Currently accepts
        'log' or 'lin'.
        '''
        if self.ax:
            if scalestring=='log':
                self.x_scale='log'
                self.AutoScale(force=True)
                try:
                    self.ax.set_xscale('log')
                    self.error_ax.set_xscale('log')
                except OverflowError:
                    self.AutoScale(force=True)
                except UserWarning:
                    pass
            elif scalestring=='linear' or scalestring=='lin':
                self.x_scale='linear'
                self.ax.set_xscale('linear')
                self.error_ax.set_xscale('linear')
                self.AutoScale(force=True)
            else:
                raise ValueError('Not allowed scaling')

            try:
                self.flush_plot()
            except UserWarning:
                pass

            self.WriteConfig(xscale=self.x_scale)
            evt=state_changed(zoomstate=self.GetZoom(),
                              yscale=self.GetYScale(), autoscale=self.autoscale,
                              xscale=self.GetXScale())
            wx.PostEvent(self.callback_window, evt)

    def create_axes(self):
        # self.ax = self.figure.add_axes(self.main_ax_rect)#
        try:
            gs=self.figure.add_gridspec(4, 1)
        except AttributeError:
            from matplotlib.gridspec import GridSpec
            gs=GridSpec(4, 1)
        self.ax=self.figure.add_subplot(gs[:3, 0])
        # self.ax.xaxis.set_visible(False)
        self.ax.get_xaxis().set_visible(False)
        # setp(self.ax.get_xticklabels(), visible=False)
        self.error_ax=self.figure.add_subplot(gs[3, 0], sharex=self.ax)
        # self.error_ax = self.figure.add_axes(self.sub_ax_rect, sharex=self.ax)
        self.ax.set_autoscale_on(False)
        self.error_ax.set_autoscale_on(True)
        self.ax.set_ylabel('Intensity [a.u.]')
        self.error_ax.set_ylabel('FOM')
        self.error_ax.set_xlabel('q [Å$^{-1}$]')

    def update_labels(self, xlabel=None, ylabel=None, elabel=None):
        if xlabel is not None:
            self.error_ax.set_xlabel(xlabel)
        if ylabel is not None:
            self.ax.set_ylabel(ylabel)
        if elabel is not None:
            self.error_ax.set_ylabel(elabel)
        self.flush_plot()

    def autoscale_error_ax(self):
        ymin=min([array(line.get_ydata()).min() \
                  for line in self.error_ax.lines if len(line.get_ydata())>0])
        ymax=max([array(line.get_ydata()).max() \
                  for line in self.error_ax.lines if len(line.get_ydata())>0])
        # Set the limits
        if ymin>=ymax:
            return
        self.error_ax.set_ylim(ymin*(1-sign(ymin)*0.05), ymax*(1+sign(ymax)*0.05))
        # self.ax.set_yscale(self.scale)

    def singleplot(self, data):
        if not self.ax:
            # self.ax = self.figure.add_subplot(111)
            self.create_axes()
        # theta = arange(0.1,10,0.001)
        # self.ax.plot(theta,1/sin(theta*pi/180)**4,'-')

    def plot_data(self, data):
        if not self.ax:
            # self.ax = self.figure.add_subplot(111)
            self.create_axes()

        # This will be somewhat inefficent since everything is updated
        # at once would be better to update the things that has changed...

        self.ax.lines=[]
        self.ax.collections=[]
        # plot the data
        # [self.ax.semilogy(data_set.x,data_set.y) for data_set in data]
        if self.scale=='linear':
            [self.ax.plot(data_set.x, data_set.y, color=data_set.data_color,
                          lw=data_set.data_linethickness, ls=data_set.data_linetype,
                          marker=data_set.data_symbol, ms=data_set.data_symbolsize) \
             for data_set in data if not data_set.use_error and data_set.show]
            # With errorbars
            [self.ax.errorbar(data_set.x, data_set.y,
                              yerr=c_[data_set.error*(data_set.error>0),
                                      data_set.error].transpose(),
                              color=data_set.data_color, lw=data_set.data_linethickness,
                              ls=data_set.data_linetype, marker=data_set.data_symbol,
                              ms=data_set.data_symbolsize) \
             for data_set in data if data_set.use_error and data_set.show]
        if self.scale=='log':
            [self.ax.plot(data_set.x.compress(data_set.y>0),
                          data_set.y.compress(data_set.y>0), color=data_set.data_color,
                          lw=data_set.data_linethickness, ls=data_set.data_linetype,
                          marker=data_set.data_symbol, ms=data_set.data_symbolsize) \
             for data_set in data if not data_set.use_error and data_set.show]
            # With errorbars
            [self.ax.errorbar(data_set.x.compress(data_set.y
                                                  -data_set.error>0),
                              data_set.y.compress(data_set.y-data_set.error>0),
                              yerr=c_[data_set.error*(data_set.error>0),
                                      data_set.error].transpose().compress(data_set.y-
                                                                           data_set.error>0),
                              color=data_set.data_color, lw=data_set.data_linethickness,
                              ls=data_set.data_linetype, marker=data_set.data_symbol,
                              ms=data_set.data_symbolsize) \
             for data_set in data if data_set.use_error and data_set.show]
        self.AutoScale()
        # Force an update of the plot
        self.flush_plot()

    def plot_data_fit(self, data):
        if not self.ax:
            # self.ax = self.figure.add_subplot(111)
            self.create_axes()

        shown_data=[data_set for data_set in data if data_set.show]
        if len(self.ax.lines)==(2*len(shown_data)):
            for i, data_set in enumerate(shown_data):
                self.ax.lines[i].set_data(data_set.x, data_set.y)
                self.ax.lines[i+len(shown_data)].set_data(data_set.x, data_set.y_sim)
                self.error_ax.lines[i].set_data(data_set.x, ma.fix_invalid(data_set.y_fom, fill_value=0))
        else:
            self.ax.lines=[]
            self.ax.collections=[]
            self.error_ax.lines=[]
            self.error_ax.collections=[]
            # plot the data
            # [self.ax.semilogy(data_set.x, data_set.y, '.'\
            # ,data_set.x, data_set.y_sim) for data_set in data]
            [self.ax.plot(data_set.x, data_set.y, color=data_set.data_color,
                          lw=data_set.data_linethickness, ls=data_set.data_linetype,
                          marker=data_set.data_symbol, ms=data_set.data_symbolsize) \
             for data_set in shown_data]
            # The same thing for the simulation
            [self.ax.plot(data_set.x, data_set.y_sim, color=data_set.sim_color,
                          lw=data_set.sim_linethickness, ls=data_set.sim_linetype,
                          marker=data_set.sim_symbol, ms=data_set.sim_symbolsize) \
             for data_set in shown_data]
            # Plot the point by point error:
            [self.error_ax.plot(data_set.x, ma.fix_invalid(data_set.y_fom, fill_value=0), color=data_set.sim_color,
                                lw=data_set.sim_linethickness, ls=data_set.sim_linetype,
                                marker=data_set.sim_symbol, ms=data_set.sim_symbolsize) \
             for data_set in shown_data]
        # Force an update of the plot
        self.autoscale_error_ax()
        self.flush_plot()

    def plot_data_sim(self, data):
        if not self.ax:
            # self.ax = self.figure.add_subplot(111)
            self.create_axes()

        p_options=[self.scale]+[[data_set.data_color, data_set.data_linethickness,
                                 data_set.data_linetype, data_set.data_symbol,
                                 data_set.data_symbolsize,
                                 data_set.sim_color, data_set.sim_linethickness,
                                 data_set.sim_linetype, data_set.sim_symbol,
                                 data_set.sim_symbolsize
                                 ] for data_set in data]
        p_datasets=[data_set for data_set in data if data_set.show]
        pe_datasets=[data_set for data_set in data if data_set.use_error and data_set.show]
        s_datasets=[data_set for data_set in data if data_set.show and data_set.use and
                                                     data_set.x.shape==data_set.y_sim.shape]
        if self._last_poptions == p_options and \
                len(self.ax.lines)==(len(p_datasets)+len(s_datasets)) and \
                len(self.ax.collections)==len(pe_datasets):
            for i, data_set in enumerate(p_datasets):
                if self.scale=='linear':
                    self.ax.lines[i].set_data(data_set.x, data_set.y)
                elif data_set.use_error:
                    fltr=(data_set.y-data_set.error)>0
                    self.ax.lines[i].set_data(data_set.x.compress(fltr), data_set.y.compress(fltr))
                else:
                    fltr=data_set.y>0
                    self.ax.lines[i].set_data(data_set.x.compress(fltr), data_set.y.compress(fltr))
            for j, data_set in enumerate(s_datasets):
                self.ax.lines[len(p_datasets)+j].set_data(data_set.x, data_set.y_sim)
                self.error_ax.lines[j].set_data(data_set.x, ma.fix_invalid(data_set.y_fom, fill_value=0))
            for k, data_set in enumerate(pe_datasets):
                ybot=data_set.y-data_set.error
                ytop=data_set.y+data_set.error
                segment_data=hstack([data_set.x, ybot, data_set.x, ytop]).reshape(2, 2, -1).transpose(2, 0, 1)
                if self.scale=='log':
                    if data_set.use_error:
                        fltr=(data_set.y-data_set.error)>0
                    else:
                        fltr=data_set.y>0
                    segment_data=segment_data[fltr, :,:]
                self.ax.collections[k].set_segments(segment_data)
        else:
            self.ax.lines=[]
            self.ax.collections=[]
            self.error_ax.lines=[]
            self.error_ax.collections=[]
            # plot the data
            # [self.ax.semilogy(data_set.x, data_set.y, '.'\
            # ,data_set.x, data_set.y_sim) for data_set in data]
            # Plot the data sets and take care if it is log scaled data

            if self.scale=='linear':
                [self.ax.plot(data_set.x, data_set.y, color=data_set.data_color,
                              lw=data_set.data_linethickness, ls=data_set.data_linetype,
                              marker=data_set.data_symbol, ms=data_set.data_symbolsize) \
                 for data_set in p_datasets if not data_set.use_error]
                # With errorbars
                [self.ax.errorbar(data_set.x, data_set.y,
                                  yerr=c_[data_set.error*(data_set.error>0),
                                          data_set.error].transpose(),
                                  color=data_set.data_color, lw=data_set.data_linethickness,
                                  ls=data_set.data_linetype, marker=data_set.data_symbol,
                                  ms=data_set.data_symbolsize) \
                 for data_set in pe_datasets]
            if self.scale=='log':
                [self.ax.plot(data_set.x.compress(data_set.y>0),
                              data_set.y.compress(data_set.y>0), color=data_set.data_color,
                              lw=data_set.data_linethickness, ls=data_set.data_linetype,
                              marker=data_set.data_symbol, ms=data_set.data_symbolsize) \
                 for data_set in p_datasets if not data_set.use_error]
                # With errorbars
                [self.ax.errorbar(data_set.x.compress(data_set.y
                                                      -data_set.error>0),
                                  data_set.y.compress(data_set.y-data_set.error>0),
                                  yerr=c_[data_set.error*(data_set.error>0),
                                          data_set.error].transpose().compress(data_set.y-
                                                                               data_set.error>0),
                                  color=data_set.data_color, lw=data_set.data_linethickness,
                                  ls=data_set.data_linetype, marker=data_set.data_symbol,
                                  ms=data_set.data_symbolsize) \
                 for data_set in pe_datasets]
            # The same thing for the simulation
            [self.ax.plot(data_set.x, data_set.y_sim, color=data_set.sim_color,
                          lw=data_set.sim_linethickness, ls=data_set.sim_linetype,
                          marker=data_set.sim_symbol, ms=data_set.sim_symbolsize) \
             for data_set in s_datasets]
            [self.error_ax.plot(data_set.x, ma.fix_invalid(data_set.y_fom, fill_value=0), color=data_set.sim_color,
                                lw=data_set.sim_linethickness, ls=data_set.sim_linetype, marker=data_set.sim_symbol,
                                ms=data_set.sim_symbolsize) for data_set in s_datasets]
            self._last_poptions=p_options
        try:
            self.autoscale_error_ax()
        except ValueError:
            pass
        self.AutoScale()
        # Force an update of the plot
        self.flush_plot()

    def OnDataListEvent(self, event):
        '''OnDataListEvent(self, event) --> None
        
        Event handler function for connection  to DataList events...
        i.e. update of the plots when the data has changed
        '''
        # print 'OnDataListEvent runs'
        # print event.data_changed, event.new_data
        data_list=event.GetData()
        if event.data_changed:
            if event.new_data:
                # print 'updating plot'
                self.update=self.plot_data
                self.update(data_list)
                tmp=self.GetAutoScale()
                self.SetAutoScale(True)
                self.AutoScale()
                self.SetAutoScale(tmp)
            else:
                self.update(data_list)
        else:
            # self.update(data_list)
            pass
        event.Skip()

    def OnSimPlotEvent(self, event):
        '''OnSimPlotEvent(self, event) --> None
        
        Event handler funciton for connection to simulation events
        i.e. update the plot with the data + simulation
        '''
        data_list=event.GetModel().get_data()
        self.update=self.plot_data_sim
        self.update(data_list)

    def OnSolverPlotEvent(self, event):
        ''' OnSolverPlotEvent(self,event) --> None
        
        Event handler function to connect to solver update events i.e.
        update the plot with the simulation
        '''
        # print 'plotting'

        if event.update_fit:
            data_list=event.model.get_data()
            if self.update!=self.plot_data_fit:
                self.update=self.plot_data_fit
                self.SetAutoScale(False)
            self.update(data_list)
        # Do not forget - pass the event on
        event.Skip()

# ==============================================================================
class ErrorPlotPanel(PlotPanel):
    ''' Class for plotting evolution of the error as a function of the
        generations.
    '''

    def __init__(self, parent, id=-1, color=None, dpi=None
                 , style=wx.NO_FULL_REPAINT_ON_RESIZE, **kwargs):
        PlotPanel.__init__(self, parent, id, color, dpi, style, **kwargs)
        self.update=self.errorplot
        self.update(None)

    def errorplot(self, data):
        if not self.ax:
            self.ax=self.figure.add_subplot(111)

        # self.ax.cla()
        self.ax.set_autoscale_on(False)

        self.ax.lines=[]
        if data is None:
            theta=arange(0.1, 10, 0.001)
            self.ax.plot(theta, floor(15-theta), '-r')
        else:
            # print 'plotting ...', data
            self.ax.plot(data[:, 0], data[:, 1], '-r')
            if self.GetAutoScale() and len(data)>0:
                self.ax.set_ylim(data[:, 1].min()*0.95, data[:, 1].max()*1.05)
                self.ax.set_xlim(data[:, 0].min(), data[:, 0].max())
                # self.AutoScale()

        self.ax.set_xlabel('Iteration')
        self.ax.set_ylabel('FOM')
        try:
            self.figure.tight_layout(h_pad=0)
        except:
            pass
        self.flush_plot()

    def OnSolverPlotEvent(self, event):
        ''' OnSolverPlotEvent(self,event) --> None
        Event handler function to connect to solver update events i.e.
        update the plot with the simulation
        '''
        # print 'error plot'
        fom_log=event.fom_log
        self.update(fom_log)
        # Do not forget - pass the event on
        event.Skip()

class ParsPlotPanel(PlotPanel):
    ''' ParsPlotPanel
    
    Class to plot the diffrent parametervalus during a fit.
    '''

    def __init__(self, parent, id=-1, color=None, dpi=None
                 , style=wx.NO_FULL_REPAINT_ON_RESIZE, **kwargs):
        PlotPanel.__init__(self, parent, id, color, dpi, style, **kwargs)
        self.update(None)
        self.ax=self.figure.add_subplot(111)
        # self.ax.set_autoscale_on(True)
        self.update=self.Plot

    def Plot(self, data):
        ''' Plots each variable and its max and min value in the
        population.
        '''

        if data.fitting:
            pop=array(data.population)
            norm=1.0/(data.max_val-data.min_val)
            best=(array(data.values)-data.min_val)*norm
            pop_min=(pop.min(0)-data.min_val)*norm
            pop_max=(pop.max(0)-data.min_val)*norm

            self.ax.cla()
            width=0.8
            x=arange(len(best))
            self.ax.set_autoscale_on(False)
            self.ax.bar(x, pop_max-pop_min, bottom=pop_min, color='b', width=width)
            self.ax.plot(x, best, 'ro')
            if self.GetAutoScale():
                self.ax.axis([x.min()-width, x.max()+width, 0., 1.])

        self.ax.set_xlabel('Parameter Index (only fittable)')
        self.ax.set_ylabel('Relative value in min/max range')
        self.figure.tight_layout(h_pad=0)
        self.flush_plot()

    def OnSolverParameterEvent(self, event):
        ''' OnSolverParameterEvent(self,event) --> None
        Event handler function to connect to solver update events i.e.
        update the plot during the fitting
        '''
        self.update(event)
        # Do not forget - pass the event on
        event.Skip()

class FomScanPlotPanel(PlotPanel):
    '''FomScanPlotPanel
    
    Class to take care of fom scans.
    '''

    def __init__(self, parent, id=-1, color=None, dpi=None
                 , style=wx.NO_FULL_REPAINT_ON_RESIZE, **kwargs):
        PlotPanel.__init__(self, parent, id, color, dpi, style, **kwargs)
        self.update(None)
        self.ax=self.figure.add_subplot(111)
        self.ax.set_autoscale_on(False)
        self.update=self.Plot

        self.type='project'

    def SetPlottype(self, type):
        '''SetScantype(self, type) --> None
        
        Sets the type of the scan type = "project" or "scan"
        '''
        if type.lower()=='project':
            self.type='project'
            self.SetAutoScale(False)
            # self.ax.set_autoscale_on(False)
        elif type.lower()=='scan':
            self.SetAutoScale(True)
            # self.ax.set_autoscale_on(True)
            self.type='scan'

    def Plot(self, data, l1='', l2=''):
        ''' Plots each variable and its max and min value in the
        population.
        '''
        self.ax.cla()
        x, y, bestx, besty, e_scale=data[0], data[1], data[2], data[3], \
                                    data[4]
        if self.type.lower()=='project':
            self.ax.set_autoscale_on(False)
            self.ax.plot(x, y, 'ob')
            self.ax.plot([bestx], [besty], 'or')
            self.ax.hlines(besty*e_scale, x.min(), x.max(), 'r')
            self.ax.axis([x.min(), x.max(), min(y.min(), besty)*0.95,
                          (besty*e_scale-min(y.min(), besty))*2.0+min(y.min(), besty)])
        elif self.type.lower()=='scan':
            self.ax.plot(x, y, 'b')
            self.ax.plot([bestx], [besty], 'or')
            self.ax.hlines(besty*e_scale, x.min(), x.max(), 'r')
            if self.GetAutoScale():
                self.ax.set_autoscale_on(False)
                self.ax.axis([x.min(), x.max(), min(y.min(), besty)*0.95, y.max()*1.05])

        self.ax.set_xlabel(l1)
        self.ax.set_ylabel(l2)

        self.flush_plot()

# =============================================================================
# Test code for the class to be able to independly test the code
if __name__=='__main__':
    class DemoPlotPanel(PlotPanel):

        def draw(self):
            if not hasattr(self, 'subplot'):
                self.subplot=self.figure.add_subplot(111)
            theta=arange(0, 45*2*pi, 0.02)
            rad=(0.8*theta/2/pi+1)
            r=rad*(8+sin(theta*7+rad/1.8))
            x=r*cos(theta)
            y=r*sin(theta)

            self.subplot.plot(x, y, '-r')

            self.subplot.set_xlim([-400, 400])
            self.subplot.set_ylim([-400, 400])

    app=wx.SimpleApp(0)
    frame=wx.Frame(None, -1, 'WxPython and Matplotlib')
    panel=DemoPlotPanel(frame)
    sizer=wx.BoxSizer(wx.HORIZONTAL)
    panel.SetSizer(sizer)
    sizer.SetItemMinSize(panel, 300, 300)
    panel.Fit()
    panel._SetSize()

    frame.Show()
    app.MainLoop()



import string

import wx
from .standard_colours import colours

class SliderControl(wx.Slider):
    """ A custom slider control."""

    def __init__(self, parent, id, value=0.0, min_value=0.0, max_value=100.0, font=None, border_frame=True,
                 size=wx.DefaultSize):
        int_value=int(100*(value-min_value)/(max_value-min_value))
        wx.Slider.__init__(self, parent, id, value=int_value, minValue=0, maxValue=100, size=size)
        self.min_value=min_value
        self.max_value=max_value
        self.value=float(value)

        self.border_frame=border_frame
        self.scroll_callback=None

    def bind_handlers(self):
        self.Bind(wx.EVT_CHAR, self.OnChar)
        self.Bind(wx.EVT_SLIDER, self.OnChange)

    def unbind_handlers(self):
        self.Unbind(wx.EVT_SLIDER)

    def OnChar(self, event):
        if chr(event.GetKeyCode())==' ':
            self.GetParent().GetParent().SaveEditControlValue()
            self.GetParent().GetParent().EnableCellEditControl(False)
            return
        else:
            event.Skip()

    def OnChange(self, event):
        self.value=self.GetValue()

        if self.scroll_callback is not None:
            self.scroll_callback(self.value)

    def set_slider_position(self):
        if self.min_value==self.max_value:
            int_value=50
        else:
            int_value=int(100*(self.value-self.min_value)/(self.max_value-self.min_value))
        wx.Slider.SetValue(self, int_value)

    def SetValue(self, value):
        self.value=value
        self.set_slider_position()

    def GetValue(self):
        int_value=wx.Slider.GetValue(self)
        return int_value/100.*(self.max_value-self.min_value)+self.min_value

    def SetMaxValue(self, value):
        self.max_value=value
        self.set_slider_position()

    def SetMinValue(self, value):
        self.min_value=value
        self.set_slider_position()

    def SetScrollCallback(self, func):
        """ Set a callback that is called on a scroll event.

        func should only take one parameter which is the value, i.e. func(value).
        """
        self.scroll_callback=func

class NumberValidator(wx.Validator):
    """Validaator to handle numerical values, accepts also 'e' as exponent value as input"""

    def __init__(self):
        wx.Validator.__init__(self)
        self.Bind(wx.EVT_CHAR, self.OnChar)

    def Clone(self):
        return NumberValidator()

    def Validate(self, win):
        tc=self.GetWindow()
        val=tc.GetValue()

        try:
            val=float(val)
        except:
            return False
        else:
            return True

    def OnChar(self, event):
        if event.AltDown() or event.ControlDown():
            event.Skip()
            return
        key=event.GetKeyCode()
        tc=self.GetWindow()
        val=tc.GetValue()
        pos=tc.GetInsertionPoint()

        if event.GetUnicodeKey()!=wx.WXK_NONE:
            # an actual character key was pressed

            if chr(key) in string.digits:
                if len(val)==0:
                    event.Skip()
                    return
                if pos==0 and val[0]=='-':
                    if not wx.Validator_IsSilent():
                        wx.Bell()
                    return
                event.Skip()
                return

            if chr(key)=='.' and '.' not in val:
                if 'e' in val:
                    # Check so that the . ends up correctly relative e
                    if pos>val.index('e'):
                        if not wx.Validator_IsSilent():
                            wx.Bell()
                        return
                if len(val)==0:
                    event.Skip()
                    return
                if pos==0 and val[0]=='-':
                    if not wx.Validator_IsSilent():
                        wx.Bell()
                    return

                event.Skip()
                return

            if chr(key)=='e' and 'e' not in val:
                if '.' in val:
                    # Check so that the e ends up correctly relative .
                    if pos<=val.index('.'):
                        if not wx.Validator_IsSilent():
                            wx.Bell()
                        return
                if len(val)==0:
                    event.Skip()
                    return
                if pos==0 and val[0]=='-':
                    if not wx.Validator_IsSilent():
                        wx.Bell()
                    return
                event.Skip()
                return

            if chr(key)=='-':
                if val[0]!='-' and pos==0:
                    event.Skip()
                elif val[pos-1]=='e' and '-' not in val[pos-1:]:
                    event.Skip()
                elif val[0]=='-':
                    tc.SetValue(val[1:])
                    tc.SetInsertionPoint(max(0,pos-1))
                else:
                    tc.SetValue('-'+val)
                    tc.SetInsertionPoint(pos+1)
                return

            if chr(key)==' ' and self.Validate(None):
                tc.GetParent().GetParent().SaveEditControlValue()
                tc.GetParent().GetParent().EnableCellEditControl(False)
                return

        if key in [wx.WXK_TAB, wx.WXK_RETURN, wx.WXK_BACK, wx.WXK_ESCAPE,
                     wx.WXK_DOWN, wx.WXK_UP, wx.WXK_LEFT, wx.WXK_RIGHT, wx.WXK_NUMPAD_LEFT,
                     wx.WXK_NUMPAD_RIGHT, wx.WXK_DELETE, wx.WXK_BACK, wx.WXK_HOME, wx.WXK_END]:
            event.Skip()
            return

        if not wx.Validator.IsSilent():
            wx.Bell()

        # Does not allow the event to propagate (kills it)
        return

class SpinCtrl(wx.TextCtrl):
    def __init__(self, parent, id=wx.NewId(), pos=wx.DefaultPosition, size=wx.DefaultSize, value=0.0,
                 min_value=None, max_value=None, steps=20, digits=6, name="SpinCtrl"):
        self.value=value
        self.digits=digits
        self.min_value=min_value
        self.max_value=max_value
        self.steps=steps
        self.value_change_func=None
        wx.TextCtrl.__init__(self, parent, id=id, pos=pos, size=size, name=name,
                             style=wx.TE_RIGHT,
                             value="%.*g"%(self.digits, self.value),
                             validator=NumberValidator())

        self.Layout()
        # self._spin=wx.SpinButton(self, wx.NewId(), pos=pos, size=wx.Size(20, self.GetSize().GetHeight()-8),
        #                          style=wx.SP_ARROW_KEYS | wx.SP_VERTICAL, name=name+"_SpinButton")
        # self._spin.Bind(wx.EVT_SPIN_UP, self.eh_spin_up,
        #                 id=self._spin.GetId())
        # self._spin.Bind(wx.EVT_SPIN_DOWN, self.eh_spin_down,
        #                 id=self._spin.GetId())

        self.Bind(wx.EVT_MOUSEWHEEL, self.OnMouseWheel)
        self.Bind(wx.EVT_CHAR, self.OnChar)

    def step(self, dir_up=True):
        """Steps the value up/down on increment

        :param dir_up: stepping direction
        :return:
        """
        inc=(self.max_value-self.min_value)*1.0/self.steps
        val=self.value
        if dir_up:
            val+=inc
        else:
            val-=inc

        # Check so that the bounds are kept
        val=max(val, self.min_value)
        val=min(val, self.max_value)

        self.SetValue(val)

        if self.value_change_func is not None:
            self.value_change_func(val)

    def eh_spin_up(self, event):
        self.step(dir_up=True)
        if self.value_change_func is not None:
            self.value_change_func(self.GetValue())
        event.Skip()

    def eh_spin_down(self, event):
        self.step(dir_up=False)
        if self.value_change_func is not None:
            self.value_change_func(self.GetValue())
        event.Skip()

    def OnMouseWheel(self, event):
        if event.GetWheelRotation()>0 and not event.IsWheelInverted():
            self.step(dir_up=True)
        else:
            self.step(dir_up=False)

    def OnChar(self, event):
        # print event.GetKeyCode()
        if event.GetKeyCode()==wx.WXK_UP:
            self.step(dir_up=True)
        elif event.GetKeyCode()==wx.WXK_DOWN:
            self.step(dir_up=False)
        else:
            event.Skip()

    def SetValue(self, value):
        try:
            self.value=float(value)
            wx.TextCtrl.SetValue(self, "%.*g"%(self.digits, self.value))
        except ValueError:
            wx.TextCtrl.SetValue(self, value)

    def GetValue(self):
        try:
            self.value=float(wx.TextCtrl.GetValue(self))
        except ValueError:
            pass
        return wx.TextCtrl.GetValue(self)

    def SetValueChangeCallback(self, func):
        """ Sets a callback function to execute when the value has changed
        :param func:
        :return:
        """
        self.value_change_func=func

    def SetRange(self, min_value, max_value):
        """ Sets the range of the control

        :param min_value:
        :param max_value:
        :return:
        """
        if max_value<min_value:
            tmp_value=min_value
            min_value=max_value
            max_value=tmp_value
        self.min_value=float(min_value)
        self.max_value=float(max_value)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
.. module:: stratified_base.py
   :synopsis: Base model for a stratified sample (Sample-Stack-Layer)

.. moduleauthor:: Matts Björck <matts.bjorck@gmail.com>

This module provides the base classes for a Sample-Stack-Layer classes. The classes implements utility functions
so that samples can be parametrised effectively.
"""

from collections import namedtuple

import numpy as np

from .parameters import Int, Float, ComplexArray, Func, Var, List, HasParameters
from .materials import Material

q=Var('q', np.array([0, 0, 0]), help="The scattering vector.")
e=Var('wl', 1.54, help="The wavelength of the radiation.")
z=Var('z', 0.0, help="The out-of-plane position in the sample.")
rep=Var('rep', 0, help="The repetition index in a Stack, starting with zero.")

validation_kwargs={'q': np.array([0, 0, 0]), 'z': 0, 'rep': 0}

class Layer(HasParameters):
    validation_kwargs=validation_kwargs
    protected=True

    def __init__(self, **kwargs):
        self.d=Float(0, unit="AA", help="Layer thickness")
        HasParameters.__init__(self, **kwargs)

class Stack(HasParameters):
    validation_kwargs=validation_kwargs
    protected=True

    def __init__(self, **kwargs):
        try:
            value=kwargs.pop('layers')
        except KeyError:
            value=[]
        self.layers=List(Layer, value=value, help="The layers in the stack")
        self.reps=Int(1, help="Number of repetitions of the stack")
        HasParameters.__init__(self, **kwargs)

    def build_layer_list(self, parameters, kwargs):
        """Builds an array for layer parameters by looping trough the stacks and layers

        Parameters:
            parameters(list of string): name of a valid layer parameters
            kwargs(dict): Keyword argument to be passed to the parameter call
        """
        out=dict()
        [out.__setitem__(par, []) for par in parameters]
        kwargs['rep']=-1

        def inner_loop(layer):
            [out[par].append(layer.__getattribute__(par)(parameter=par, **kwargs)) for par in parameters]
            kwargs['z']+=layer.d(**kwargs)

        def outer_loop(rep):
            kwargs['rep']+=1
            list(map(inner_loop, self.layers))

        list(map(outer_loop, list(range(self.reps(**kwargs)))))

        kwargs.pop('rep')
        return out

class Sample(HasParameters):
    validation_kwargs=validation_kwargs

    def __init__(self, **kwargs):
        try:
            value=kwargs.pop('stacks')
        except KeyError:
            value=[]
        self.stacks=List(Stack, value=value, help="The stacks in the sample")
        # self.substrate = Layer()
        # self.ambient = Layer()

        # dictionary that holds the last evaluated layer parameters - used for checking the calcs
        self.layer_dic={}

        HasParameters.__init__(self, **kwargs)

    def build_layer_list(self, parameters, kwargs):
        """Builds an array for layer parameters by looping through the stacks and layers

        Parameters:
            parameters(list of string): name of a valid layer parameter
            kwargs(dict): Keyword argument to be passed to the parameter call

        Returns:
            ret: namedtuple with the parameters as members, for example ret.d
        """
        kwargs['z']=0.0

        out=dict()
        [out.__setitem__(par, [self.substrate.__getattribute__(par)(**kwargs)]) for par in parameters]
        for stack in self.stacks:
            stack_dict=stack.build_layer_list(parameters, kwargs)
            [out[par].extend(stack_dict[par]) for par in parameters]
        kwargs['z']=0.0
        [out[par].append(self.ambient.__getattribute__(par)(parameter=par, **kwargs)) for par in parameters]
        Collection=namedtuple('layer_parameters', list(out.keys()))
        return Collection(**out)

class Instrument(HasParameters):
    validation_kwargs=validation_kwargs

    def __init__(self, **kwargs):
        HasParameters.__init__(self, **kwargs)



from numpy import *
from functools import reduce
from genx.gui_logging import iprint

# "Ordinary" implementaion of Parrats recursion formula
# theta-vector, lamda- can be a vector,n-1Dvector, d-1Dvector, sigma-1Dvector
def Refl(theta, lamda, n, d, sigma, return_int=True):
    d=d[1:-1]
    sigma=sigma[:-1]
    # Length of k-vector in vaccum
    k=2*math.pi/lamda
    # Calculates the wavevector in each layer
    Qj=2*n[-1]*k*sqrt(n[:, newaxis]**2/n[-1]**2-cos(theta*math.pi/180)**2)
    # Fresnel reflectivity for the interfaces
    rp=(Qj[1:]-Qj[:-1])/(Qj[1:]+Qj[:-1])*exp(-Qj[1:]*Qj[:-1]/2*sigma[:, newaxis]**2)
    # Ignoring the top and bottom layer for the calc.
    p=exp(1.0j*d[:, newaxis]*Qj[1:-1])
    # Setting up a matrix for the reduce function. Reduce only takes one array
    # as argument
    rpp=array(list(map(lambda x, y: [x, y], rp[1:], p)))

    # Paratt's recursion formula
    def formula(rtot, rint):
        return (rint[0]+rtot*rint[1])/(1+rtot*rint[0]*rint[1])

    # Implement the recursion formula
    r=reduce(formula, rpp, rp[0])

    if return_int:
        return abs(r)**2
    else:
        return r

# "Ordinary" implementaion of Parrats recursion formula
# Q-vector,n-1Dvector, d-1Dvector, sigma-1Dvector
def ReflQ(Q, lamda, n, d, sigma, return_int=True):
    # Length of k-vector in vaccum
    d=d[1:-1]
    sigma=sigma[:-1]
    Q0=4*pi/lamda
    Q=Q.astype(complex128)
    # Calculates the wavevector in each layer
    Qj=sqrt((n[:, newaxis]**2-n[-1]**2)*Q0**2+n[-1]**2*Q**2)
    # Fresnel reflectivity for the interfaces
    rp=(Qj[1:]-Qj[:-1])/(Qj[1:]+Qj[:-1])*exp(-Qj[1:]*Qj[:-1]/2*sigma[:, newaxis]**2)
    # print rp.shape #For debugging
    # print d.shape
    # print Qj[1:-1].shape
    p=exp(1.0j*d[:, newaxis]*Qj[1:-1])  # Ignoring the top and bottom layer for the calc.
    # print p.shape #For debugging
    # Setting up a matrix for the reduce function. Reduce only takes one array
    # as argument
    rpp=array(list(map(lambda x, y: [x, y], rp[1:], p)))

    # print rpp.shape
    # Paratt's recursion formula
    def formula(rtot, rint):
        return (rint[0]+rtot*rint[1])/(1+rtot*rint[0]*rint[1])

    # Implement the recursion formula
    r=reduce(formula, rpp, rp[0])
    # print r.shape
    # return the reflectivity 
    if return_int:
        return abs(r)**2
    else:
        return r

# Parrats recursion formula for varying n given by n_func-function
def Refl_nvary(theta, lamda, n_func, d, sigma, return_int=True):
    d=d[1:-1]
    sigma=sigma[:-1]
    # Length of k-vector in vaccum
    k=2*math.pi/lamda
    # Calculates the wavevector in each layer
    # print n_func
    ss=transpose((sin(theta[:, newaxis]*pi/180.0)/lamda)*ones(len(n_func)))
    # print ss.shape
    # print theta.shape
    # print len(n_func)
    n=array(list(map(lambda f, val: f(val), n_func, ss)))
    # print n
    Qj=2*k*n[-1]*sqrt(n**2/n[-1]**2-cos(theta*math.pi/180)**2)
    # Fresnel reflectivity for the interfaces
    rp=(Qj[1:]-Qj[:-1])/(Qj[1:]+Qj[:-1])*exp(-Qj[1:]*Qj[:-1]/2*sigma[:, newaxis]**2)
    # print rp.shape #For debugging
    # print d.shape
    # print Qj[1:-1].shape
    p=exp(1.0j*d[:, newaxis]*Qj[1:-1])  # Ignoring the top and bottom layer for the calc.
    # print p.shape #For debugging
    # Setting up a matrix for the reduce function. Reduce only takes one array
    # as argument
    rpp=array(list(map(lambda x, y: [x, y], rp[1:], p)))

    # print rpp.shape
    # Paratt's recursion formula
    def formula(rtot, rint):
        return (rint[0]+rtot*rint[1])/(1+rtot*rint[0]*rint[1])

    # Implement the recursion formula
    r=reduce(formula, rpp, rp[0])
    if return_int:
        return abs(r)**2
    else:
        return r

def ReflProfiles(theta, lamda, n, d, sigma, profile, return_int=True):
    d=d[1:-1]
    sigma=sigma[:-1]
    # Length of k-vector in vaccum
    k=2*math.pi/lamda
    # Calculates the wavevector in each layer
    Qj=2*n[-1]*k*sqrt(n[:, newaxis]**2/n[-1]**2-cos(theta*math.pi/180)**2)

    # Function to map caclulate the roughness values
    def w(QiQj, sigma, profile):
        # erf profile
        if profile==0:
            return exp(-QiQj*sigma**2/2)
        # exponential profile
        elif profile==1:
            return 1/(1+sqrt(QiQj)*sigma**2/2)
        # linear profile
        elif profile==2:
            return sin(sqrt(3)*sigma*sqrt(QiQj))/sqrt(3)/sigma/sqrt(QiQj)
        else:
            # If any wrong number are typed in return the erf profile
            return exp(-QiQj*sigma**2/2)

    interface=array(list(map(w, Qj[:-1]*Qj[1:], sigma, profile)))
    # Fresnel reflectivity for the interfaces
    rp=(Qj[1:]-Qj[:-1])/(Qj[1:]+Qj[:-1])*interface
    # print rp.shape #For debugging
    # print d.shape
    # print Qj[1:-1].shape
    p=exp(1.0j*d[:, newaxis]*Qj[1:-1])  # Ignoring the top and bottom layer for the calc.
    # print p.shape #For debugging
    # Setting up a matrix for the reduce function. Reduce only takes one array
    # as argument
    rpp=array(list(map(lambda x, y: [x, y], rp[1:], p)))

    # print rpp.shape
    # Paratt's recursion formula
    def formula(rtot, rint):
        return (rint[0]+rtot*rint[1])/(1+rtot*rint[0]*rint[1])

    # Implement the recursion formula
    r=reduce(formula, rpp, rp[0])
    # print r.shape
    # return the reflectivity 
    if return_int:
        return abs(r)**2
    else:
        return r

# paratts algorithm for n as function of lamda or theta
def Refl_nvary2(theta, lamda, n_vector, d, sigma, return_int=True):
    d=d[1:-1]
    sigma=sigma[:-1]
    # Length of k-vector in vaccum
    k=2*math.pi/lamda
    # Calculates the wavevector in each layer
    # print n_func
    # ss=transpose((sin(theta[:,newaxis]*pi/180.0)/lamda)*ones(len(n_func)))
    # print ss.shape
    # print theta.shape
    # print len(n_func)
    # n=array(map(lambda f,val:f(val),n_func,ss))
    n=n_vector
    # print n
    Qj=2*n[-1]*k*sqrt(n**2/n[-1]**2-cos(theta*math.pi/180)**2)
    # print sigma.shape, Qj.shape
    # Fresnel reflectivity for the interfaces
    rp=(Qj[1:]-Qj[:-1])/(Qj[1:]+Qj[:-1])*exp(-Qj[1:]*Qj[:-1]/2*sigma[:, newaxis]**2)
    # print rp.shape #For debugging
    # print d.shape
    # print Qj[1:-1].shape
    p=exp(1.0j*d[:, newaxis]*Qj[1:-1])  # Ignoring the top and bottom layer for the calc.
    # print p.shape #For debugging
    # Setting up a matrix for the reduce function. Reduce only takes one array
    # as argument
    rpp=array(list(map(lambda x, y: [x, y], rp[1:], p)))

    # print rpp.shape
    # Paratt's recursion formula
    def formula(rtot, rint):
        return (rint[0]+rtot*rint[1])/(1+rtot*rint[0]*rint[1])

    # Implement the recursion formula
    r=reduce(formula, rpp, rp[0])
    if return_int:
        return abs(r)**2
    else:
        return r

def reflq_kin(q, lamda, n, d, sigma, correct_q=True, return_int=True):
    """Calculates the reflectivity in the kinematical approximation"""
    d=d[:-1]
    d[0]=0
    z=d.sum()-d.cumsum()
    sigma=sigma[:-1]
    q0=4*pi/lamda
    # Kinematical reflectivity for the interfaces
    iprint(n.shape, len(n.shape))
    if len(n.shape)==1:
        if correct_q:
            # The internal wave vector calacuted with the thickness averaged refractive index.
            n_mean=(n[1:-1]*d[1:]/d.sum()).sum()
            q_corr=sqrt((n_mean**2-n[-1]**2)*q0**2+(n[-1]*q)**2)
        else:
            q_corr=q
        rp=(n[:-1]-n[1:])[:, newaxis]*exp(-(q_corr*sigma[:, newaxis])**2/2)
    else:
        if correct_q:
            # The internal wave vector calacuted with the thickness averaged refractive index.
            n_mean=(n[1:-1]*d[1:][:, newaxis]/d.sum()).sum(axis=0)
            q_corr=sqrt((n_mean**2-n[-1]**2)*q0**2+(n[-1]*q)**2)
        else:
            q_corr=q
        rp=(n[:-1]-n[1:])*exp(-(q_corr*sigma[:, newaxis])**2/2)
    p=exp(1.0j*z[:, newaxis]*q_corr)

    r=(rp*p).sum(axis=0)*q0**2/q_corr**2/2.

    if return_int:
        return abs(r)**2
    else:
        return r

def reflq_pseudo_kin(q, lamda, n, d, sigma, return_int=True):
    """Calculates the reflectivity in a pseudo kinematical approximation.
    The mean refractive index of the film is simulated with the single reflection approximation and the deviation from
    the mean is simulated with the kinematical approximation.
    """
    d=d[:-1]
    d[0]=0
    z=d.sum()-d.cumsum()
    sigma=sigma[:-1]
    q0=4*pi/lamda
    # Q = Q.astype(complex128)
    # The internal wave vector calacuted with the thickness averaged refractive index.
    n_mean=(n[1:-1]*d[1:]/d.sum()).sum()
    q_corr=sqrt((n_mean**2-n[-1]**2)*q0**2+n[-1]**2*q**2)
    q_sub=sqrt((n[0]**2-n[-1]**2)*q0**2+n[-1]**2*q**2)
    q_amb=n[-1]*q
    # Top interface
    rp_top=(q_corr-q_amb)/(q_corr+q_amb)*exp(-q_corr*q/2*sigma[-1]**2)
    rp_sub=(q_sub-q_corr)/(q_sub+q_corr)*exp(-q_sub*q_corr/2*sigma[0]**2)
    # rp_top = -(n[-1] - n_mean)*exp(-(q_corr*sigma[-1])**2/2)*q0**2/q_corr**2/2.
    # rp_sub = -(n_mean - n[0])*exp(-(q_corr*sigma[0])**2/2)*q0**2/q_corr**2/2.
    # Kinematical reflectivity for the interfaces
    n_diff=n-n_mean
    n_diff[0]=0
    n_diff[-1]=0
    rp=(n_diff[:-1]-n_diff[1:])[:, newaxis]*exp(-(q_corr*sigma[:, newaxis])**2/2)
    p=exp(1.0j*z[:, newaxis]*q_corr)

    r_kin=(rp*p).sum(axis=0)*q0**2/q_corr**2/2.
    r_sra=rp_top+rp_sub*exp(1.0j*d.sum()*q_corr)
    r=r_kin+r_sra

    if return_int:
        return abs(r)**2
    else:
        return r

def reflq_sra(q, lamda, n, d, sigma, return_int=True):
    """Single reflection approximation calculation of the reflectivity"""
    # Length of k-vector in vaccum
    d=d[1:-1]
    sigma=sigma[:-1]
    q0=4*pi/lamda
    # Calculates the wavevector in each layer
    if len(n.shape)==1:
        qj=sqrt((n[:, newaxis]**2-n[-1]**2)*q0**2+(n[-1]*q)**2)
    else:
        qj=sqrt((n**2-n[-1]**2)*q0**2+(n[-1]*q)**2)
    # Fresnel reflectivity for the interfaces
    rp=(qj[:-1]-qj[1:])/(qj[1:]+qj[:-1])*exp(-qj[1:]*qj[:-1]/2*sigma[:, newaxis]**2)
    # The wave does not transverse the ambient and substrate - ignoring them
    # Also, the wave travels from top -> bottom, the array has the first element as the substrate
    # - need to reverse the order.
    phaseterm=(d[:, newaxis]*qj[1:-1])[::-1].cumsum(axis=0)[::-1]
    p=exp(1.0J*phaseterm)
    # Adding the first interface (top -> last in array) since p is not calculated for that layer (p = 1)
    r=rp[-1]+(rp[:-1]*p).sum(axis=0)
    if return_int:
        return abs(r)**2
    else:
        return r

from . import USE_NUMBA

if USE_NUMBA:
    # try to use numba to speed up the calculation intensive functions:
    try:
        from .paratt_numba import Refl, ReflQ, Refl_nvary2
    except Exception as e:
        iprint('Could not use numba, no speed up from JIT compiler:\n'+str(e))

if __name__=='__main__':
    theta=arange(0, 5, 0.01)+1e-13
    # c=paratt.Refl(theta,1.54,array([1-7.57e-6+1.73e-7j,1-2.24e-5+2.89e-6j,1-7.57e-6+1.73e-7j,1-2.24e-5+2.89e-6j,1-7.57e-6+1.73e-7j,1-2.24e-5+2.89e-6j,1]),array([80,20,80,20,80,20]),0)
    # c1=Refl(theta,1.54,array([1-7.57e-6+1.73e-7j,1-2.24e-5+2.89e-6j,1-7.57e-6+1.73e-7j,1-2.24e-5+2.89e-6j,1-7.57e-6+1.73e-7j,1-2.24e-5+2.89e-6j,1-7.57e-6+1.73e-7j,1]),array([80,20,80,20,80,20]),4)
    c2=Refl(theta, 1.54, array(
        [1-7.57e-6+1.73e-7j, 1-2.24e-5+2.89e-6j, 1-7.57e-6+1.73e-7j, 1-2.24e-5+2.89e-6j, 1-7.57e-6+1.73e-7j,
         1-2.24e-5+2.89e-6j, 1-7.57e-6+1.73e-7j, 1]), array([2, 80, 20, 80, 20, 80, 20, 2]),
            array([0, 0, 0, 0, 0, 0, 0, 0]))
    # print c.shape
    # gplt.plot(theta,log10(c1),theta,log10(c2))
    # gplt.plot(theta,log10(c2))



''' help.py - A module that includes various help functionality 
for genx. 

First is the ModelsHelpDialog which displays the doc strings of each module
in the models directory.
'''

import wx, os
import wx.html as  html
from . import event_handlers as eh

try:
    from docutils.core import publish_doctree, publish_from_doctree
    from docutils.parsers.rst import roles
except ImportError:
    def rst_html(text):
        return "For proper display install docutils.<\br>\n"+text
else:
    def _role_fn(name, rawtext, text, lineno, inliner, options=None, content=None):
        if options is None:
            options={}
        if content is None:
            content=[]
        return [], []

    roles.register_canonical_role('mod', _role_fn)

    def rst_html(text):
        return publish_from_doctree(publish_doctree(text), writer_name='html').decode()

class ExampleHandler:
    '''A class to handle the examples bundled with GenX
    '''

    def __init__(self, parent, example_menu, path):
        ''' Inits the Handler,
        
        parent is the main window
        path is the path to the example folder
        example_menu is the menu item that should contain the examples
        '''
        self.parent=parent
        self.path=path
        self.menu=example_menu

        self.update_menu()

    def update_menu(self):
        ''' Updates the list of examples'''
        # Remove all items present in the submenu
        items=self.menu.GetMenuItems()
        [self.menu.DeleteItem(item) for item in items]

        examples=self.get_examples()
        examples.sort()

        # Add new menu items
        for ex in examples:
            menu=self.menu.Append(-1, ex)
            self.parent.Bind(wx.EVT_MENU, self.LoadExample, menu)

    def get_examples(self):
        examples=[s[:-3] for s in os.listdir(self.path) if '.gx'==s[-3:]]
        return examples

    def LoadExample(self, event):
        menuitem=self.menu.FindItemById(event.GetId())
        example=menuitem.GetLabel()
        path=self.path+example+'.gx'
        eh.open_model(self.parent, path)

class PluginHelpDialog(wx.Frame):
    def __init__(self, parent, module, title='Models help'):
        wx.Frame.__init__(self, parent, -1, title)
        # self.SetAutoLayout(True)
        self.module=module
        self.sub_modules=True

        sizer=wx.BoxSizer(wx.VERTICAL)
        choice_sizer=wx.BoxSizer(wx.HORIZONTAL)
        choice_sizer.Add(wx.StaticText(self, -1, 'Module: '), 0, wx.CENTER)
        mod_list=self.find_modules(module)

        self.choice=wx.Choice(self, -1, choices=mod_list)
        choice_sizer.Add(self.choice, 0, flag=wx.EXPAND | wx.CENTER,
                         border=20)
        self.Bind(wx.EVT_CHOICE, self.on_choice, self.choice)
        sizer.Add(choice_sizer, 0, wx.EXPAND, border=20)

        self.html_win=html.HtmlWindow(self, -1,
                                      style=wx.NO_FULL_REPAINT_ON_RESIZE)
        sizer.Add(self.html_win, 1, flag=wx.EXPAND, border=20)

        self.SetSizer(sizer)

        sizer.Fit(self)
        self.Layout()
        size=parent.GetSize()
        x=max(size.y/2, size.x/3)
        self.SetSize((x, size.y))

        self.choice.SetSelection(0)
        self.on_choice(None)

    def on_choice(self, event):
        '''on_choice(self, event) --> None
        
        Callback for a choice selction in the choice window
        '''
        sub_module=self.choice.GetStringSelection()
        if self.sub_modules:
            doc=self.load_doc(self.module, sub_module)
        else:
            doc=self.load_doc(self.module)
        self.html_win.SetPage(doc)

    def find_modules(self, module):
        '''find_modules(self, directory) --> string_list
        
        Finds all modules in a directory. Only finds .py files and not files
        beginning _
        '''
        # Load the package, note the non-empty fromlist that 
        # makes subpackages being loaded
        mod=__import__(module, globals(), locals(), [''])
        try:
            modules=[s[:-3] for s in os.listdir(mod.__path__[0]) \
                     if s[0]!='_' and s[-3:]=='.py']
        except AttributeError:
            modules=[module]
            self.sub_modules=False
        return modules

    def load_doc(self, module, sub_module=None):
        '''load_doc(self, module, sub_moduls) --> doc string
        
        Returns the doc string for module.sub_module list
        '''
        docs=''
        try:
            if sub_module is not None:
                mod=__import__('%s.%s'%(module, sub_module),
                               globals(), locals(), [''])
            else:
                mod=__import__('%s'%module,
                               globals(), locals(), [''])
        except Exception as e:
            docs='Could not load docstring for %s.'%sub_module
            docs+='\n The following exception occured: %s'%str(e)
        else:
            if mod.__doc__ is None:
                docs="No documentation available for module."
            elif '<h1>' in mod.__doc__:
                docs=mod.__doc__
            else:
                docs=rst_html(mod.__doc__)
        if type(docs)!=type(''):
            docs='The doc string is of the wrong type in module %s'%sub_module
        return docs



'''
=====================================================
:mod:`sns_mr` SNS magnetism reflectometer data loader
=====================================================

Loads the default datafile format used for extracted reflectivity at
the SNS magnetism reflectometer. It allows to import several channels at
once, automatically naming the datasets according to the imported
polarizations.
'''

import numpy as np

from ..data_loader_framework import Template
from ..utils import ShowWarningDialog

class Plugin(Template):
    wildcard='*.dat'

    def __init__(self, parent):
        Template.__init__(self, parent)
        self.x_col=0
        self.y_col=1
        self.e_col=2
        self.xe_col=3
        self.ai_col=4
        self.comment='#'
        self.skip_rows=0
        self.delimiter=None

    def CanOpen(self, file_path):
        if not Template.CanOpen(self, file_path):
            return False
        l1=open(file_path, 'r', encoding='utf-8').readline()
        return l1.startswith('# Datafile created by QuickNXS')

    def LoadData(self, dataset, filename):
        '''LoadData(self, data_item_number, filename) --> none

        Loads the data from filename into the data_item_number.
        '''
        # get scan number and polarization channel from filename
        name=''
        header={'Date': '', 'Type': '', 'Input file indices': '', 'Extracted states': ''}
        fhandle=open(filename, 'r', encoding='utf-8')
        fline=fhandle.readline()
        while not '[Data]' in fline:
            if ':' in fline:
                key, value=map(str.strip, fline[2:].split(':', 1))
                print(key, value)
                if key=='Input file indices':
                    name+=value
                elif key=='Extracted states':
                    name+=' (%s)'%(value.split(' ')[0])
                if key in header:
                    header[key]=value
            fline=fhandle.readline()
            if not fline:
                ShowWarningDialog(self.parent, 'Could not load the file: '+filename+' \nWrong format.\n')
                return

        try:
            load_array=np.loadtxt(fhandle, delimiter=self.delimiter, comments=self.comment, skiprows=self.skip_rows)
        except Exception as e:
            ShowWarningDialog(self.parent, 'Could not load the file: '+filename+
                              ' \nPlease check the format.\n\n numpy.loadtxt'+' gave the following error:\n'+str(e))
            return
        else:
            # For the freak case of only one data point
            if len(load_array.shape)<2:
                load_array=np.array([load_array])
            # Check so we have enough columns
            if load_array.shape[1]-1<max(self.x_col, self.y_col, self.e_col):
                ShowWarningDialog(self.parent, 'The data file does not contain'+'enough number of columns. It has '
                                  +str(load_array[1])+' columns. Rember that the column index start at zero!')
                # Okay now we have showed a dialog lets bail out ...
                return
            # The data is set by the default Template.__init__ function, neat hu
            # Know the loaded data goes into *_raw so that they are not
            # changed by the transforms
            lamda=4.*np.pi/load_array[:, self.x_col]*np.sin(load_array[:, self.ai_col])
            dataset.x_raw=load_array[:, self.x_col]
            dataset.y_raw=load_array[:, self.y_col]
            dataset.error_raw=load_array[:, self.e_col]
            dataset.set_extra_data('res', load_array[:, self.xe_col], 'res')
            dataset.set_extra_data('ai', load_array[:, self.ai_col], 'ai')
            dataset.set_extra_data('wavelength', lamda, 'wavelength')
            # Name the dataset accordign to file name
            dataset.name=name
            # Run the commands on the data - this also sets the x,y, error memebers
            # of that data item.
            dataset.run_command()

            # insert metadata into ORSO compatible fields
            dataset.meta['data_source']['facility']='SNS@ORNL'
            dataset.meta['data_source']['experimentDate']=header['Date']
            dataset.meta['data_source']['experiment']['instrument']='MagRef (4A)'
            dataset.meta['data_source']['experiment']['probe']='neutron'
            dataset.meta['data_source']['measurement']['scheme']='energy-dispersive'
            dataset.meta['data_source']['measurement']['omega']={'min': float(load_array[:, self.ai_col].min()),
                                                                 'max': float(load_array[:, self.ai_col].max()),
                                                                 'unit': 'rad'}
            dataset.meta['data_source']['measurement']['wavelength']={'min': float(lamda.min()),
                                                                      'max': float(lamda.max()),
                                                                      'unit': 'angstrom'}
            dataset.meta['reduction']={'software': {'name': 'QuickNXS',
                                                    'file_indices': header['Input file indices'],
                                                    'spin_states': header['Extracted states']}}



'''
Library for specular and off-specular x-ray reflectivity
========================================================
interdiff is a model for specular and off specular simulations including
the effects of interdiffusion in hte calculations. The specular
simulations is conducted with Parrats recursion formula. The
off-specular, diffuse calculations are done with the distorted Born wave
approximation (DWBA) as derived by Holy and with the extensions done by
Wormington to include diffuse interfaces.

Classes
-------

Layer
~~~~~
``Layer(b = 0.0, d = 0.0, f = 0.0+0.0J, dens = 1.0, magn_ang = 0.0, magn = 0.0, sigma = 0.0)``

``d``
   The thickness of the layer in AA (Angstroms = 1e-10m)
``f``
   The x-ray scattering length per formula unit in electrons. To be
   strict it is the number of Thompson scattering lengths for each
   formula unit.
``dens``
   The density of formula units in units per Angstroms. Note the units!
``sigmai``
   The root mean square *interdiffusion* of the top interface of the
   layer in Angstroms.
``sigmar``
   The root mean square *roughness* of the top interface of the layer in
   Angstroms.

Stack
~~~~~
``Stack(Layers = [], Repetitions = 1)``

``Layers``
   A ``list`` consiting of ``Layer``\ s in the stack the first item is
   the layer closest to the bottom
``Repetitions``
   The number of repsetions of the stack

Sample
~~~~~~
``Sample(Stacks = [], Ambient = Layer(), Substrate = Layer(), eta_z = 10.0,     eta_x = 10.0, h = 1.0)``

``Stacks``
   A ``list`` consiting of ``Stack``\ s in the stacks the first item is
   the layer closest to the bottom
``Ambient``
   A ``Layer`` describing the Ambient (enviroment above the sample).
   Only the scattering lengths and density of the layer is used.
``Substrate``
   A ``Layer`` describing the substrate (enviroment below the sample).
   Only the scattering lengths, density and roughness of the layer is
   used.
``eta_z``
   The out-of plane (vertical) correlation length of the roughness in
   the sample. Given in AA.
``eta_x``
   The in-plane global correlation length (it is assumed equal for all
   layers). Given in AA.
``h``
   The jaggedness parameter, should be between 0 and 1.0. This describes
   how jagged the interfaces are. This is also a global parameter for
   all interfaces.

Instrument
~~~~~~~~~~
``Instrument(wavelength = 1.54, coords = 'tth',      I0 = 1.0 res = 0.001, restype = 'no conv', respoints = 5, resintrange = 2,      beamw = 0.01, footype = 'no corr', samplelen = 10.0, taylor_n = 1)``

``wavelength``
    The wavalelngth of the radiation givenin AA (Angstroms)
``coords``
    The coordinates of the data given to the SimSpecular function. The
    available alternatives are: 'q' or 'tth'. Alternatively the numbers 0
    (q) or 1 (tth) can be used.
``I0``
    The incident intensity (a scaling factor)
``Ibkg``
    The background intensity. Added as a constant value to the calculated
    reflectivity
``res``
    The resolution of the instrument given in the coordinates of ``coords``.
    This assumes a gaussian reloution function and ``res`` is the standard
    deviation of that gaussian.
``restype``
    Describes the rype of the resolution calculated. One of the
    alterantives: 'no conv', 'fast conv', 'full conv and varying res.' or
    'fast conv + varying res.'. The respective numbers 0-3 also works. Note
    that fast convolution only alllows a single value into res wheras the
    other can also take an array with the same length as the x-data (varying
    resolution)
``respoints``
    The number of points to include in the resolution calculation. This is
    only used for 'full conv and vaying res.' and 'fast conv + varying res'
``resintrange``
    Number of standard deviatons to integrate the resolution fucntion times
    the relfectivty over
``footype``
    Which type of footprint correction is to be applied to the simulation.
    One of: 'no corr', 'gauss beam' or 'square beam'. Alternatively, the
    number 0-2 are also valid. The different choices are self explanatory.
``beamw``
    The width of the beam given in mm. For 'gauss beam' it should be the
    standard deviation. For 'square beam' it is the full width of the beam.
``samplelen``
    The length of the sample given in mm
``taylor_n``
    The number terms taken into account in the taylor expansion of the
    fourier integral of the correlation function. More terms more accurate
    calculation but also much slower.
'''
from genx.gui_logging import iprint
from .lib import paratt as Paratt
from .lib import offspec

from numpy import *
from scipy.special import erf
from .lib.instrument import *

# Preamble to define the parameters needed for the models outlined below:
ModelID='MingInterdiff'
# Automatic loading of parameters possible by including this list
__pars__=['Layer', 'Stack', 'Sample', 'Instrument']
# Used for making choices in the GUI
instrument_string_choices={'coords': ['q', 'tth'],
                           'restype': ['no conv', 'fast conv',
                                       'full conv and varying res.', 'fast conv + varying res.'],
                           'footype': ['no corr', 'gauss beam', 'square beam']}

InstrumentParameters={'wavelength': 1.54, 'coords': 'tth', 'I0': 1.0, 'res': 0.001,
                      'restype': 'no conv', 'respoints': 5, 'resintrange': 2, 'beamw': 0.01, 'footype': 'no corr',
                      'samplelen': 10.0, 'Ibkg': 0.0, 'taylor_n': 1}
# Coordinates=1 => twothetainput
# Coordinates=0 => Q input
# Res stddev of resolution
# ResType 0: No resolution convlution
#               1: Fast convolution
#               2: Full Convolution +varying resolution
#               3: Fast convolution varying resolution
# ResPoints Number of points for the convolution only valid for ResolutionType=2
# ResIntrange Number of standard deviatons to integrate over default 2
# Parameters for footprint coorections
# Footype: 0: No corections for footprint
#          1: Correction for Gaussian beam => Beaw given in mm and stddev
#          2: Correction for square profile => Beaw given in full width mm
# Samlen= Samplelength in mm.

LayerParameters={'sigmai': 0.0, 'sigmar': 0.0, 'dens': 1.0, 'd': 0.0,
                 'f': 0.0+0.0j}
StackParameters={'Layers': [], 'Repetitions': 1}
SampleParameters={'Stacks': [], 'Ambient': None, 'Substrate': None, 'h': 1.0,
                  'eta_z': 10.0, 'eta_x': 10.0}

def Specular(TwoThetaQz, sample, instrument):
    ''' Simulate the specular signal from sample when proped with instrument
    
    # BEGIN Parameters
    TwoThetaQz data.x
    # END Parameters
    '''
    # preamble to get it working with my class interface
    restype=instrument.getRestype()

    if restype==2 or restype==instrument_string_choices['restype'][2]:
        (TwoThetaQz, weight)=ResolutionVector(TwoThetaQz[:],
                                              instrument.getRes(), instrument.getRespoints(),
                                              range=instrument.getResintrange())
    if instrument.getCoords()==1 or \
            instrument.getCoords()==instrument_string_choices['coords'][1]:
        theta=TwoThetaQz/2
    elif instrument.getCoords()==0 or \
            instrument.getCoords()==instrument_string_choices['coords'][0]:
        theta=arcsin(TwoThetaQz/4/pi*instrument.getWavelength())*180./pi

    lamda=instrument.getWavelength()
    parameters=sample.resolveLayerParameters()
    dens=array(parameters['dens'], dtype=float64)
    # print [type(f) for f in parameters['f']]
    f=array(parameters['f'], dtype=complex128)
    re=2.82e-13*1e2/1e-10
    n=1-dens*re*lamda**2/2/pi*f*1e-4
    d=array(parameters['d'], dtype=float64)
    # d = d[1:-1]
    sigmar=array(parameters['sigmar'], dtype=float64)
    # sigmar = sigmar[:-1]
    sigmai=array(parameters['sigmai'], dtype=float64)
    # sigmai = sigmai[:-1]
    sigma=sqrt(sigmai**2+sigmar**2)
    # print sigma

    R=Paratt.Refl(theta, lamda, n, d, sigma)*instrument.getI0()

    # FootprintCorrections

    foocor=1.0
    footype=instrument.getFootype()
    beamw=instrument.getBeamw()
    samlen=instrument.getSamplelen()
    if footype==0 or footype==instrument_string_choices['footype'][0]:
        foocor=1.0
    elif footype==1 or footype==instrument_string_choices['footype'][1]:
        foocor=GaussIntensity(theta, samlen/2.0, samlen/2.0, beamw)
    elif footype==2 or footype==instrument_string_choices['footype'][2]:
        foocor=SquareIntensity(theta, samlen, beamw)
    else:
        raise ValueError('Variable footype has an unvalid value')

    if restype==0 or restype==instrument_string_choices['restype'][0]:
        R=R[:]*foocor
    elif restype==1 or restype==instrument_string_choices['restype'][1]:
        R=ConvoluteFast(TwoThetaQz, R[:]*foocor, instrument.getRes(),
                        range=instrument.getResintrange())
    elif restype==2 or restype==instrument_string_choices['restype'][2]:
        R=ConvoluteResolutionVector(TwoThetaQz, R[:]*foocor, weight)
    elif restype==3 or restype==instrument_string_choices['restype'][3]:
        R=ConvoluteFastVar(TwoThetaQz, R[:]*foocor, instrument.getRes(),
                           range=instrument.getResintrange())
    else:
        raise ValueError('Variable restype has an unvalid value')
    return R+instrument.getIbkg()

def OffSpecularMingInterdiff(TwoThetaQz, ThetaQx, sample, instrument):
    ''' Function that simulates the off-specular signal (not implemented)
    
    # BEGIN Parameters
    TwoThetaQz 1.0
    ThetaQx data.x
    # END Parameters
    '''
    lamda=instrument.getWavelength()
    if instrument.getCoords() in [1, instrument_string_choices['coords'][1]]:
        alphaR1=ThetaQx
        betaR1=TwoThetaQz-ThetaQx
        qx=2*pi/lamda*(cos(alphaR1*pi/180)-cos(betaR1*pi/180))
        qz=2*pi/lamda*(sin(alphaR1*pi/180)+sin(betaR1*pi/180))
    else:
        qz=TwoThetaQz
        qx=ThetaQx

    parameters=sample.resolveLayerParameters()

    def toarray(a, code):
        a=list(a)
        a.reverse()
        return array(a, dtype=code)

    dens=array(parameters['dens'], dtype=complex64)
    f=array(parameters['f'], dtype=complex64)
    re=2.82e-13*1e2/1e-10
    n=1-dens*re*lamda**2/2/pi*f*1e-4
    n=toarray(n, code=complex64)
    sigmar=toarray(parameters['sigmar'], code=float64)
    sigmar=sigmar[1:]

    sigmai=toarray(parameters['sigmai'], code=float64)
    sigmai=sigmai[1:]+1e-5

    d=toarray(parameters['d'], code=float64)
    d=r_[0, d[1:-1]]

    z=-cumsum(d)

    eta=sample.getEta_x()

    h=sample.getH()

    eta_z=sample.getEta_z()

    (I, alpha, omega)=offspec.DWBA_Interdiff(qx, qz, lamda, n, z,
                                                 sigmar, sigmai, eta, h, eta_z, d,
                                                 taylor_n=instrument.getTaylor_n())

    restype=instrument.getRestype()
    if restype==0 or restype==instrument_string_choices['restype'][0]:
        # if no resolution is defined, don't include specular peak
        return real(I)*instrument.getI0()+instrument.getIbkg()

    # include specular peak
    instrument.setRestype(0)
    if isinstance(TwoThetaQz, ndarray):
        Ispec=Specular(TwoThetaQz, sample, instrument)
    else:
        Ispec=Specular(array([TwoThetaQz], dtype=float64), sample, instrument)[0]
    instrument.setRestype(restype)

    if instrument.getCoords() in [1, instrument_string_choices['coords'][1]]:
        spec_peak=Ispec*exp(-0.5*(TwoThetaQz/2.-ThetaQx)**2/instrument.getRes()**2)
    else:
        spec_peak=Ispec*exp(-0.5*ThetaQx**2/instrument.getRes()**2)
    return (spec_peak+real(I))*instrument.getI0()+instrument.getIbkg()

def SLD_calculations(z, item, sample, inst):
    ''' Calculates the scatteringlength density as at the positions z
    
    # BEGIN Parameters
    z data.x
    item "Re"
    # END Parameters
    '''
    parameters=sample.resolveLayerParameters()
    dens=array(parameters['dens'], dtype=complex64)
    f=array(parameters['f'], dtype=complex64)
    sld=dens*f
    d_sld=sld[:-1]-sld[1:]
    d=array(parameters['d'], dtype=float64)
    d=d[1:-1]
    # Include one extra element - the zero pos (substrate/film interface)
    int_pos=cumsum(r_[0, d])
    sigmar=array(parameters['sigmar'], dtype=float64)
    sigmar=sigmar[:-1]
    sigmai=array(parameters['sigmai'], dtype=float64)
    sigmai=sigmai[:-1]
    sigma=sqrt(sigmai**2+sigmar**2)+1e-7
    if z is None:
        z=arange(-sigma[0]*5, int_pos.max()+sigma[-1]*5, 0.5)
    rho=sum(d_sld*(0.5-0.5*erf((z[:, newaxis]-int_pos)/sqrt(2.)/sigma)), 1)+sld[-1]
    dic={'Re': real(rho), 'Im': imag(rho), 'z': z,
         'SLD unit': 'r_{e}/\AA^{3}'}
    if item is None or item=='all':
        return dic
    else:
        try:
            return dic[item]
        except:
            raise ValueError('The chosen item, %s, does not exist'%item)

SimulationFunctions={'Specular': Specular,
                     'OffSpecular': OffSpecularMingInterdiff,
                     'SLD': SLD_calculations}

from .lib import refl as Refl

(Instrument, Layer, Stack, Sample)=Refl.MakeClasses(InstrumentParameters,
                                                    LayerParameters, StackParameters,
                                                    SampleParameters, SimulationFunctions, ModelID)

if __name__=='__main__':
    Fe=Layer(d=10, sigmar=3.0, n=1-2.247e-5+2.891e-6j)
    Si=Layer(d=15, sigmar=3.0, n=1-7.577e-6+1.756e-7j)
    sub=Layer(sigmar=3.0, n=1-7.577e-6+1.756e-7j)
    amb=Layer(n=1.0, sigmar=1.0)
    stack=Stack(Layers=[Fe, Si], Repetitions=20)
    sample=Sample(Stacks=[stack], Ambient=amb, Substrate=sub, eta_z=500.0, eta_x=100.0)
    iprint(sample)
    inst=Instrument(Wavelength=1.54, Coordinates=1)



'''
Library for surface x-ray diffraction simulations of superlattices
==================================================================

The model is based on Fullertons algorithm for superlattices as
described in Phys. Rev. B vol. 45 p. 9292 (1992). 
'''
# Programmed by Matts Bjorck 20091215

import numpy as np
from . import sxrd
from .utils import f, rho
import time

from .sxrd import UnitCell, AtomGroup, Instrument, Slab, SymTrans

__pars__=['SLSample', 'SLStandard', 'UnitCell', 'Slab', 'AtomGroup',
          'Instrument']

class SLSample:
    '''Class that models a multilayer sample on top of a 
    substrate according to Fullertons model as given in
    PRB ....
    '''

    def __init__(self, inst, bulk_slab, superlattice, unitcell,
                 bulk_sym=None):
        if bulk_sym is None:
            bulk_sym=[]
        self.set_bulk_slab(bulk_slab)
        self.set_bulk_sym(bulk_sym)
        self.superlattice=superlattice
        self.inst=inst
        self.set_unit_cell(unitcell)

    def set_bulk_slab(self, bulk_slab):
        '''Set the bulk unit cell to bulk_slab
        '''
        if type(bulk_slab)!=type(sxrd.Slab()):
            raise TypeError("The bulk slab has to be a member of"
                            " class Slab")
        self.bulk=bulk_slab

    def set_unit_cell(self, unit_cell):
        '''Sets the unitcell of the sample
        '''
        if type(unit_cell)!=type(sxrd.UnitCell(1.0, 1.0, 1.0)):
            raise TypeError("The bulk slab has to be a member"
                            " of class UnitCell")
        if unit_cell is None:
            unit_cell=sxrd.UnitCell(1.0, 1, .0, 1.0)
        self.unit_cell=unit_cell

    def set_bulk_sym(self, sym_list):
        '''Sets the list of allowed symmetry operations for the bulk

        sym_list has to be a list ([]) of symmetry elements from the
        class SymTrans
        '''
        # Type checking
        if type(sym_list)!=type([]):
            raise TypeError("The surface symmetries has to contained"
                            " in a list")

        if not sym_list:
            sym_list=[sxrd.SymTrans()]

        if min([type(sym)==type(sxrd.SymTrans()) for
                sym in sym_list])==0:
            raise TypeError("All members in the symmetry list has to"
                            " be a memeber of class SymTrans")

        self.bulk_sym=sym_list

    def calc_i(self, h, k, l):
        '''Calculate the diffracted intensity from a superlattice.
        The diffracted intensity from the superlattice and the substrate
        are added. I.e. it is assumed that the films is not coherent with
        the substrate.
        '''
        bulk_i=np.abs(self.calc_fb(h, k, l))**2
        sl_i=np.abs(self.superlattice.calc_i(h, k, l))
        return (bulk_i+sl_i)*self.inst.inten

    def calc_fb(self, h, k, l):
        '''Calculate the structure factors from the bulk
        '''
        dinv=self.unit_cell.abs_hkl(h, k, l)
        x, y, z, el, u, oc, c=self.bulk._extract_values()
        oc=oc/float(len(self.bulk_sym))
        f=sxrd._get_f(self.inst, el, dinv)
        # Calculate the "shape factor" for the CTRs
        eff_thick=self.unit_cell.c/np.sin(self.inst.alpha*np.pi/180.0)
        alpha=(2.82e-5*self.inst.wavel*eff_thick/self.unit_cell.vol()*
               np.sum(f.imag, 1))
        denom=np.exp(2.0*np.pi*1.0J*l)*np.exp(-alpha)-1.0
        # Delta functions to remove finite size effect in hk plane
        delta_funcs=(abs(h-np.round(h))<1e-12)*(
                abs(k-np.round(k))<1e-12)
        # Sum up the uc struct factors
        f_u=np.sum(oc*f*np.exp(-2*np.pi**2*u*dinv[:, np.newaxis]**2)*
                   np.sum([np.exp(2.0*np.pi*1.0J*(
                           h[:, np.newaxis]*sym_op.trans_x(x, y)+
                           k[:, np.newaxis]*sym_op.trans_y(x, y)+
                           l[:, np.newaxis]*z[np.newaxis, :]))
                           for sym_op in self.bulk_sym], 0)
                   , 1)
        # Putting it all togheter
        fb=f_u/denom*delta_funcs

        return fb

class Superlattice:
    '''Class that describe a superlattice, can be subclassed
    to implement different strain profiles, interdiffusion etc..
    '''

    def __init__(self, inst, unit_cell, a_slab, b_slab,
                 a_sym=None, b_sym=None):
        if a_sym is None:
            a_sym=[]
        if b_sym is None:
            b_sym=[]
        self.a_slab=a_slab
        self.b_slab=b_slab
        if not a_sym:
            self.a_sym=[sxrd.SymTrans()]
        else:
            self.a_sym=a_sym
        if not b_sym:
            self.b_sym=[sxrd.SymTrans()]
        else:
            self.b_sym=b_sym
        self.unit_cell=unit_cell
        self.inst=inst

    def _extract_slab_values(self, slabs, sym):
        '''Extracts the necessary parameters for simulating
        a list of stacked slabs
        '''
        # Extract the parameters we need
        # the star in zip(*... transform the list elements to arguments
        xt, yt, zt, elt, ut, oct, ct=list(zip(*[slab._extract_values()
                                                for slab in slabs]))

        x=np.r_[xt]
        y=np.r_[yt]
        # scale and shift the slabs with respect to each other
        cn=np.cumsum(np.r_[0, ct])[:-1]
        z=np.concatenate([zs*c_s+c_cum
                          for zs, c_cum, c_s in zip(zt, cn, ct)])
        # el = reduce(lambda x,y:x+y, elt)
        el=np.r_[elt]
        u=np.r_[ut]
        oc=np.r_[oct]
        # print x,y,z, u
        t_lay=sum(ct)
        return x, y, z, u, oc, el, t_lay

    def calc_fslab(self, slablist, sym, h, k, l):
        '''Calculate the structure factors from the bulk
        '''
        dinv=self.unit_cell.abs_hkl(h, k, l)
        x, y, z, u, oc, el, t_lay=self._extract_slab_values(slablist,
                                                            sym)
        oc=oc/float(len(sym))
        f=sxrd._get_f(self.inst, el, dinv)
        # Sum up the uc struct factors
        f_u=np.sum(oc*f*np.exp(-2*np.pi**2*u*dinv[:, np.newaxis]**2)*
                   np.sum([np.exp(2.0*np.pi*1.0J*(
                           h[:, np.newaxis]*sym_op.trans_x(x, y)+
                           k[:, np.newaxis]*sym_op.trans_y(x, y)+
                           l[:, np.newaxis]*z[np.newaxis, :]))
                           for sym_op in sym], 0)
                   , 1)
        # return f_u, (z.max() - z.min())*np.ones(l.shape)
        return f_u, t_lay*np.ones(l.shape)

    def calc_fa(self, n, h, k, l):
        '''Calculate the strucutre factor for a a layer
        n is the thickness of the bilayer in units of slabs'''
        pass

    def calc_fb(self, n, h, k, l):
        '''Calcualte the structure factor for a b layer
        n is the thickness of the bilayer in units of slabs'''
        pass

    def calc_fsl(self, unit_cell, h, k, l):
        '''Calculate the strucutre factor for the entire 
        superlattice.
        '''
        raise NotImplementedError('calc_fsl has to be implemented in '
                                  'a Superlattices subclass')

class SLStandard(Superlattice):
    '''Class that implements a "standard" superlattice, no strain 
    included.
    '''
    _pars={'sigmaa': 1e-12, 'sigmab': 1e-12, 'repetitions': 2, 'na': 2,
           'nb': 2, 'a': 0.0, 'c': 1e-12}

    def __init__(self, inst, unit_cell, a_slab, b_slab,
                 a_sym=None, b_sym=None):
        Superlattice.__init__(self, inst, unit_cell, a_slab, b_slab,
                              a_sym=a_sym, b_sym=b_sym)
        if a_sym is None:
            a_sym=[]
        if b_sym is None:
            b_sym=[]
        [self._make_set_func(name, self._pars[name]) for name in
         list(self._pars.keys())]
        [self._make_get_func(name) for name in list(self._pars.keys())]

    def calc_fa(self, n, h, k, l):
        f_slab, t_z=self.calc_fslab([self.a_slab]*n, self.a_sym,
                                    h, k, l)
        return f_slab, t_z

    def calc_fb(self, n, h, k, l):
        f_slab, t_z=self.calc_fslab([self.b_slab]*n, self.b_sym,
                                    h, k, l)
        return f_slab, t_z

    def thick_prob(self, n_mean, stand_dev):
        # According to fullerton it's enough to include three
        # standard deviations in the averaging
        lower=np.floor(n_mean-3.0*stand_dev)
        lower=np.int(min(lower, n_mean-1))
        # We can't have a negative thickness, altough we remove
        # the gaussian distribution How does this affect the theoretical
        # assumptions?
        if lower<1:
            lower=1
        upper=np.ceil(n_mean+3.0*stand_dev)
        n=np.arange(lower,
                    np.int(max(upper, n_mean+1)+1))
        # print 'n: ', n
        prob=np.exp(-(n-n_mean)**2/2.0/stand_dev**2)
        prob=prob/sum(prob)
        return n, prob

    def calc_i(self, h, k, l):
        ''' Function to calculate the form factor from a superlattice
        '''
        # Create the different thicknesses to avarage over
        na, pa=self.thick_prob(self.na, self.sigmaa)
        nb, pb=self.thick_prob(self.nb, self.sigmab)
        tmp=np.array([self.calc_fa(n, h, k, l) for n in na])
        fa=tmp[:, 0, :]
        ta=tmp[:, 1, :]
        tmp=np.array([self.calc_fb(n, h, k, l) for n in nb])
        fb=tmp[:, 0, :]
        tb=tmp[:, 1, :]
        # print pa.shape, fa.shape
        pa=pa[:, np.newaxis]
        pb=pb[:, np.newaxis]
        # Do the different averagning
        fafa=(pa*fa*fa.conj()).sum(0)
        fbfb=(pb*fb*fb.conj()).sum(0)
        fam=(pa*fa).sum(0)
        fbm=(pb*fb).sum(0)
        phia=(pa*np.exp(2*np.pi*1.0J*ta*l)*fa.conj()).sum(0)
        phib=(pb*np.exp(2*np.pi*1.0J*tb*l)*fb.conj()).sum(0)
        ta=(pa*np.exp(2*np.pi*1.0J*ta*l)).sum(0)
        tb=(pb*np.exp(2*np.pi*1.0J*tb*l)).sum(0)
        m=self.repetitions
        ksi=2*np.pi*1.0J*l*self.a-(2*np.pi*l*self.c)**2/2.
        # Calculate the intensity
        int=(m*(fafa+2.0*np.real(np.exp(ksi)*phia*fbm)+fbfb)+
             2.0*np.real((np.exp(-ksi)*phib*fam/ta/tb+phia*fam/ta+
                          phib*fbm/tb+np.exp(ksi)*phia*fbm)*(
                                 (m-(m+1)*np.exp(2.0*ksi)*ta*tb+
                                  (np.exp(2.0*ksi)*ta*tb)**(m+1))/
                                 (1-np.exp(2.0*ksi)*ta*tb)**2-m)))
        return int

    def _make_set_func(self, name, val):
        '''Creates a function to set value for attribute with name'''

        def set_func(value):
            setattr(self, name, value)

        # Init the variable
        set_func(val)
        setattr(self, 'set'+name, set_func)

    def _make_get_func(self, name):
        ''' Creates a get function '''

        def get_func():
            return getattr(self, name)

        setattr(self, 'get'+name, get_func)

if __name__=='__main__':
    from pylab import *

    inst=sxrd.Instrument(wavel=0.77, alpha=0.2)
    inst.set_inten(100.0)

    lay_a=sxrd.Slab()
    lay_a.add_atom('Sr', 'sr', 0.0, 0.0, 0.0, 0.001, 1.0)
    lay_a.add_atom('Ti', 'ti', 0.5, 0.5, 0.5, 0.001, 1.0)
    lay_a.add_atom('O1', 'o', 0.5, 0.0, 0.5, 0.001, 1.0)
    lay_a.add_atom('O2', 'o', 0.0, 0.5, 0.5, 0.001, 1.0)
    lay_a.add_atom('O3', 'o', 0.5, 0.5, 0.0, 0.001, 1.0)

    lay_b=sxrd.Slab(c=1.0)
    lay_b.add_atom('La', 'la', 0.0, 0.0, 0.0, 0.001, 1.0, 1.0)
    lay_b.add_atom('Al', 'al', 0.5, 0.5, 0.5, 0.001, 1.0, 1.0)
    lay_b.add_atom('O1', 'o', 0.5, 0.5, 0.0, 0.001, 1.0, 1.)
    lay_b.add_atom('O2', 'o', 0.0, 0.5, 0.5, 0.001, 1.0, 1.)
    lay_b.add_atom('O3', 'o', 0.5, 0.0, 0.5, 0.001, 1.0, 1.)

    uc=sxrd.UnitCell(3.945, 3.945, 3.945, 90, 90, 90)

    sl=SLStandard(inst, uc, lay_b, lay_a)
    sl_sample=SLSample(inst, lay_a, sl, uc)
    sl.seta(0.0)
    sl.setc(0.00001)
    sl.setna(4.0)
    # Seems to have a lower limit of about 0.3 UC to work fine
    # with the calculation of thicknesses..
    sl.setsigmaa(0.3)
    sl.setnb(2.0)
    sl.setsigmab(0.3)
    sl.setrepetitions(10)
    l=np.arange(0.1, 3, 0.0011)
    h=0.0*np.ones(l.shape)
    k=0.0*np.ones(l.shape)
    int=sl_sample.calc_i(h, k, l)

    sample=sxrd.Sample(inst, lay_a, ([lay_b]*4+[lay_a]*2)*10,
                       sxrd.UnitCell(3.945, 3.945, 3.945, 90, 90, 90))
    f_ref=sample.calc_f(h, k, l)
    int_ref=abs(f_ref)**2
    # Comparison between the normal sxrd model and the superlattice model.
    semilogy(l, int_ref)
    semilogy(l, int)
    legend(('sxrd model', 'sxrd_mult closed form'))
    xlabel('l [r.l.u.]')
    ylabel('Intensity')
    show()
